#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_p $opt_q  $opt_o);
&getopts('hi:j:p:q:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   an organism name (e.g. hsa)
# -j   an organism name (e.g. mmu)
# -p   a directory containing files containing a list of scalar values
# -q   a directory containing files containing a list of scalar values
# -o   a directory for output

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $org1    = $opt_i or die $usage;
my $org2    = $opt_j or die $usage;
my $dir1    = $opt_p or die $usage;
my $dir2    = $opt_q or die $usage;
my $out_dir = $opt_o or die $usage;
$dir1      .= '/' unless $dir1 =~ /\/$/;
$dir2      .= '/' unless $dir2 =~ /\/$/;
$out_dir   .= '/' unless $out_dir =~ /\/$/;

my $all_f1  = $dir1 . $org1 . '.vs.' . $org2 . '.all.dist';
my $in__f1  = $dir1 . $org1 . '.vs.' . $org2 . '.in.dist';
my $out_f1  = $dir1 . $org1 . '.vs.' . $org2 . '.out.dist';
my $all_f2  = $dir2 . $org1 . '.vs.' . $org2 . '.all.dist';
my $in__f2  = $dir2 . $org1 . '.vs.' . $org2 . '.in.dist';
my $out_f2  = $dir2 . $org1 . '.vs.' . $org2 . '.out.dist';
    
my $jobname = $out_dir . $org1 . '.vs.' . $org2 . '.INvsOUT';
my $dat_f1  = $jobname . '.1.dat';
my $dat_f2  = $jobname . '.2.dat';
my $gnufile = $jobname . '.gnu';
my $epsfile = $jobname . '.eps';

# Load input files
my @in_val1 = read_value_file( $in__f1 );
my @outval1 = read_value_file( $out_f1 );
my @in_val2 = read_value_file( $in__f2 );
my @outval2 = read_value_file( $out_f2 );

# Draw 2d plot
print_data( $dat_f1, \@in_val1, \@in_val2 );
print_data( $dat_f2, \@outval1, \@outval2 );
my $num_in  = wc( $dat_f1 );
my $num_out = wc( $dat_f2 );
print STDERR $num_in  . " data points were printed : " . $dat_f1 . "\n";
print STDERR $num_out . " data points were printed : " . $dat_f2 . "\n";
print_gnu_file( $dat_f1, $dat_f2, $gnufile, $epsfile );
`gnuplot $gnufile`;


sub read_value_file {
    my $file = shift;
    my @vals;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if( @cols == 1 ){ push( @vals, $cols[0] ); }
    }
    close IN;
    return @vals;
}


sub print_data {
    my $file   = shift;
    my $vals_1 = shift;
    my $vals_2 = shift;
    open(DAT, ">$file") or die "Can't open $file to write.";
    if( @$vals_1 != @$vals_2 ){
	die "Conflicting number of values:";
    }
    for( my $i=0 ; $i<@$vals_1 ; ++$i ){
	my $val1 = $$vals_1[$i];
	my $val2 = $$vals_2[$i];
	unless( ($val1 eq 'N/A') or ($val2 eq 'N/A') ){
	    print DAT $val1 . "\t" . $val2 . "\n";
	}
    }
    close DAT;
}


sub wc {
    my $file = shift;
    my $wc   = `wc -l $file`;
    my @wc   = split(" ", $wc);
    return $wc[0];
}


sub print_gnu_file {
    my $dat_f1  = shift;
    my $dat_f2  = shift;
    my $gnufile = shift;
    my $epsfile = shift;
    open(GNU, ">$gnufile") or die "Can't open $gnufile to write.";
    print GNU 'set terminal postscript color'                         . "\n"
	. 'set output "' . $epsfile . '"'                             . "\n"
	. 'plot "' . $dat_f1 . '" with points, '
	. '"' . $dat_f2 . '" with points'                             . "\n";
    close GNU;
}
