#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_p $opt_s $opt_d);
&getopts('hi:j:p:s:d:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   an organism name 1 (e.g. hsa)
# -j   an organism name 2 (e.g. mmu)
# -p   a file containing gene pairs (e.g. *.rbh)
# -s   a directory containing protein sequence files 
# -d   a directory containing dna sequence files

## Others:
# -h   print help

_EOH_
    ;

die $usage if $opt_h;

## internal parameters -- start
my $kmed="YN00";
$kmed=$opt_k if $opt_k;
die "$kmed is not appropriate calculation method."
    unless ($kmed eq "NG86") or ($kmed eq "YN00") or ($kmed eq "LWL85")
    or ($kmed eq "LWL85m") or ($kmed eq "LPB93");
## internal parameters -- end

# Set parameters
my $org1    = $opt_i or die $usage;
my $org2    = $opt_j or die $usage;
my $paifile = $opt_p or die $usage;
my $pep_dir = $opt_s or die $usage;
my $dna_dir = $opt_d or die $usage;
my $outfile = $paifile . '.kaks';
$pep_dir   .= '/' unless $pep_dir =~ /\/$/;
$dna_dir   .= '/' unless $dna_dir =~ /\/$/;
my $pep_f1  = $pep_dir . $org1 . '.mfa';
my $pep_f2  = $pep_dir . $org2 . '.mfa';
my $dna_f1  = $dna_dir . $org1 . '.mfa';
my $dna_f2  = $dna_dir . $org2 . '.mfa';
my $tmp_dir = '/tmp/';
my $gapopen = 10.0;
my $gapextend = 0.5;

# Read input files
my %g2pep_1 = read_mfa_file( $pep_f1 );
my $p_num_1 = keys %g2pep_1;
print STDERR $p_num_1 . ' protein sequences were found in ' . $pep_f1 . "\n";
my %g2pep_2 = read_mfa_file( $pep_f2 );
my $p_num_2 = keys %g2pep_2;
print STDERR $p_num_2 . ' protein sequences were found in ' . $pep_f2 . "\n";
my %g2dna_1 = read_mfa_file( $dna_f1 );
my $d_num_1 = keys %g2dna_1;
print STDERR $d_num_1 . ' DNA sequences were found in ' . $dna_f1 . "\n";
my %g2dna_2 = read_mfa_file( $dna_f2 );
my $d_num_2 = keys %g2dna_2;
print STDERR $d_num_2 . ' DNA sequences were found in ' . $dna_f2 . "\n";
my %pairs   = read_pairs_file( $paifile );
my $p_num   = keys %pairs;
print STDERR $p_num . ' gene pairs were found in ' . $paifile . "\n";

open(PAMDIST, ">$outfile") or die "Can't open $outfile to write.";
# Compute global alignment via the needle program
foreach my $pair ( sort keys %pairs ){
    my ($g1, $g2) = split("\t", $pair);
    my ($p_aln1, $p_aln2) 
	= execute_needle( $pair, \%g2pep_1, \%g2pep_2, $tmp_dir );
    my $d_aln1 = pepAln2dnaAln( $p_aln1, $g2dna_1{$g1} );
    my $d_aln2 = pepAln2dnaAln( $p_aln2, $g2dna_2{$g2} );
    if( $d_aln1 eq 'N/A' ){print STDERR $g1 . "\n"; next;} 
    if( $d_aln2 eq 'N/A' ){print STDERR $g2 . "\n"; next;} 
    my $pam = calc_pamdist( $pair, $d_aln1, $d_aln2, $tmp_dir );
    # print PAMDIST $pair . "\t" . $pam . "\n";
    print PAMDIST $pam . "\n" if $pam>=0;
    print $pam . "\n" if $pam>=0;
}
close PAMDIST;



# Load input files
my %pair2info = read_oasys_results( $infile, $sttID, $endID );
my $num_pair = keys %pair2info;
print "$num_pair orthologous gene pairs were found.\n";
my %gid2pep = read_mfa_file( $pepfile );
my $num_pep = keys %gid2pep;
print $num_pep . ' protein sequences were found in ' . $pepfile . "\n";
my %gid2dna = read_mfa_file( $dnafile );
my $num_dna = keys %gid2dna;
print $num_dna . ' coding DNA sequences were found in ' . $dnafile . "\n";
my %gid2chr = read_pos_file( $posfile );
my $num_chr = keys %gid2chr;
print $num_chr . ' genes were found in ' . $posfile . "\n";

# Compute Ka/Ks values
print "Calculating Ka/Ks values...\n";
my $j;
my $numPair=keys %pair2info;
foreach my $pair (sort keys %pair2info){
    ++$j;

    my ($gid1,$gid2)=split(" ",$pair);
    next if !defined $gid2pep{$gid1};
    next if !defined $gid2pep{$gid2};
    next if !defined $gid2dna{$gid1};
    next if !defined $gid2dna{$gid2};

    print "($j/$numPair)\nCalculating Ka/Ks values for $gid1 vs $gid2 ...\n";
    my $tmpPepfile1="/home/hpc/hacchy/study/ortholog/tmp/$gid1.pep.fa";
    open(TMP,">$tmpPepfile1") or die "Can't open $tmpPepfile1 to write.";
    print TMP ">$gid1\n$gid2pep{$gid1}\n";
    close TMP;
    my $tmpPepfile2="/home/hpc/hacchy/study/ortholog/tmp/$gid2.pep.fa";
    open(TMP,">$tmpPepfile2") or die "Can't open $tmpPepfile2 to write.";
    print TMP ">$gid2\n$gid2pep{$gid2}\n";
    close TMP;

    my $needle="/home/hpc/hacchy/study/ortholog/EMBOSS-5.0.0/emboss/needle";
    my $tmpNeedlefile="/home/hpc/hacchy/study/ortholog/tmp/$gid1-$gid2.needle";
    my $param="-asequence $tmpPepfile1 -bsequence $tmpPepfile2 -gapopen 10.0 -gapextend 0.5 -outfile $tmpNeedlefile";
    `$needle $param`;

    my $tmpdnafile="/home/hpc/hacchy/study/ortholog/tmp/$gid1-$gid2.cdna.fa";
    open(TMP,">$tmpdnafile") or die "Can't open $tmpdnafile to write.";
    print TMP ">$gid1\n$gid2dna{$gid1}\n";
    print TMP ">$gid2\n$gid2dna{$gid2}\n";
    close TMP;

    my $srspair2nuc="/home/hpc/hacchy/study/ortholog/scripts/srspair2nuc.pl";
    my $tmpNucfile="/home/hpc/hacchy/study/ortholog/tmp/$gid1-$gid2.nuc";
    my ($flg_m,$flg_M);
    if($gid2chr{$gid1} eq 'MT'){
	$flg_m=1;
    } else {
	$flg_m=0;
    }
    if($gid2chr{$gid2} eq 'MT'){
	$flg_M=1;
    } else {
	$flg_M=0;
    }
    
    my $KaKs;
    if($flg_m == $flg_M){
	`perl -w $srspair2nuc -i $tmpNeedlefile -c $tmpdnafile -o $tmpNucfile -m $flg_m`;
	my ($numLine)=split(" ",`wc -l $tmpNucfile`);
	next if $numLine==0;

	my $yn00="/home/hpc/hacchy/study/ortholog/paml4/bin/yn00";
	my $yn00_ctl="/home/hpc/hacchy/study/ortholog/tmp/$gid1-$gid2.yn00.ctl";
	my $tmpYnfile="/home/hpc/hacchy/study/ortholog/tmp/$gid1-$gid2.yn00.result";
	open(CTL,">$yn00_ctl") or die "Can't open $yn00_ctl to write.";
	print CTL "seqfile    = $tmpNucfile\n";
	print CTL "outfile    = $tmpYnfile\n";
	print CTL "verbose    = 0\n";
	print CTL "icode      = $flg_m\n";
	print CTL "weighting  = 0\n";
	print CTL "commonf3x4 = 0\n";
	print CTL "ndata      = 1\n";
	close CTL;
	`$yn00 $yn00_ctl`;

	my $parseYn00="/home/hpc/hacchy/study/ortholog/scripts/parseYn00.pl";
	my $tmpSumfile="/home/hpc/hacchy/study/ortholog/tmp/$gid1-$gid2.yn00.summary";
	`perl -w $parseYn00 -i $tmpYnfile -o $tmpSumfile`;
	
	`rm $tmpPepfile1`;
	`rm $tmpPepfile2`;
	`rm $tmpNeedlefile`;
	`rm $tmpdnafile`;
	`rm $tmpNucfile`;
	`rm $yn00_ctl`;
	`rm $tmpYnfile`;

	open(YN,$tmpSumfile) or die "Can't open $tmpSumfile\n";
	my $tmp=<YN>;
	while(<YN>){
	    chomp;
	    my @col=split(" ",$_);
	    if(@col==5){
		my ($method,$val)=($col[0],$col[2]);
		# $KaKs=$val if $method eq $kmed;
		$KaKs .= "\t" . $val;
	    }
	}
	close YN;

	`rm $tmpSumfile`;
    } else {
	$KaKs='MT';
    }

    print "\t$KaKs";
    $pair2info{$gid1." ".$gid2} .= "$KaKs";
    print "\n\n";

    print OUT $pair2info{$gid1." ".$gid2},"\n";
}
print "...done\n";
print "$j pairs' Ka/Ks values were calculated.\n\n";
close OUT;


sub read_oasys_results {
    my $file  = shift;
    my $sttID = shift;
    my $endID = shift;
    my %pair2info;
    open(IN, $file) or die "Can't open $file to read.";
    my $cur_id = 0;
    while(<IN>){
	chomp;
	my @col=split(" ",$_);
	next if @col<2;
	my ($gid1, $gid2) = @col;
	my $pair=$gid1." ".$gid2;
	if( $cur_id >= $sttID and $cur_id < $endID ){
	    $pair2info{$pair}=$_;
	}
	++$cur_id;
    }
    close IN;
    return %pair2info;
}


sub read_mfa_file {
    my $file = shift;
    my %gid2seq;
    open(MFA, $file) or die "Can't open $file to read.";
    my ($gid,$seq);
    while(<MFA>){
	chomp;
	if(/^>(\S+)/){
	    if(defined $seq){
		$gid2seq{$gid}=$seq;
	    }
	    $gid = $1;
	    undef $seq;
	} else {
	    $_=~s/\s+//g;
	    $seq.=$_;
	}
    }
    if(defined $seq){
	$gid2seq{$gid}=$seq;
    }
    close MFA;
    return %gid2seq;
}


sub read_pos_file {
    my $file = shift;
    my %gid2chr;
    open(POS, $file) or die "Can't open $file to read.";
    while(<POS>){
	chomp;
	my @cols = split("\t", $_);
	if( @cols == 5 ){
	    my ($gid, $chr) = @cols[0, 1];
	    $gid2chr{$gid}  = $chr;
	}
    }
    close POS;
    return %gid2chr;
}
