#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_o);
&getopts('hi:j:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing RBH pairs (*.rbh)
# -j   another file containing RBH pairs (*.rbh)
# -o   output file

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $rbhfile1 = $opt_i or die $usage;
my $rbhfile2 = $opt_j or die $usage;
my $outfile  = $opt_o or die $usage;

# Read input files
my %list_1 = read_rbh_file( $rbhfile1 );
my %list_2 = read_rbh_file( $rbhfile2 );

# Calculate overlapping rate
my %op_list = overlap( \%list_1, \%list_2 );
my $num_1   = keys %list_1;
my $num_2   = keys %list_2;
my $num_op  = keys %op_list;
my $opr_1   = round( $num_op/$num_1, 4 );
my $opr_2   = round( $num_op/$num_2, 4 );

# Output results
open(OUT, ">$outfile") or die "Can't open $outfile to write.";
print OUT 
    "Number of orthologs in first set  : " . $num_1  . "\n" .
    "Number of orthologs in second set : " . $num_2  . "\n" .
    "Number of overlaping orthologs    : " . $num_op . "\n" . 
    "Overlapping rate 1                : " . $opr_1  . "\n" .
    "Overlapping rate 2                : " . $opr_2  . "\n";
print STDERR
    "Number of orthologs in first set  : " . $num_1  . "\n" .
    "Number of orthologs in second set : " . $num_2  . "\n" .
    "Number of overlaping orthologs    : " . $num_op . "\n" . 
    "Overlapping rate 1                : " . $opr_1  . "\n" .
    "Overlapping rate 2                : " . $opr_2  . "\n";

close OUT;


sub read_rbh_file {
    my $file = shift;
    my %list;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split(" ", $_);
	if(@cols==3){
	    my $rbh = $cols[0] . "\t" . $cols[1];
	    $list{$rbh} = 1;
	}
    }
    close IN;
    return %list;
}


sub overlap {
    my $list_1 = shift;
    my $list_2 = shift;
    my %list;
    foreach my $gp (sort keys %$list_1){
	$list{$gp} = 1 if( $$list_2{$gp} );
    }
    return %list;
}


sub round {
    my $val = shift;
    my $col = shift;
    my $r   = 10**$col;
    my $a   = ($val>0) ? 0.5 : -0.5;
    return int($val*$r+$a)/$r;
}
