#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_n $opt_o);
&getopts('hi:j:n:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing the statistics of the DPD clustering (*.cluster.stats)
# -j   a file containing the statistics of the DPD clustering (*.cluster.stats)
# -o   output name

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $statsFile_1 = $opt_i or die $usage;
my $statsFile_2 = $opt_j or die $usage;
my $outName     = $opt_o or die $usage;

# Set file names
my $datFile_1   = $outName.'.1.dat';
my $datFile_2   = $outName.'.2.dat';

# Read cluster stats file
my %size2freq_1 = read_stats_file( $statsFile_1 );
my %size2freq_2 = read_stats_file( $statsFile_2 );

# Print data 
print_data( $datFile_1, \%size2freq_1 );
print_data( $datFile_2, \%size2freq_2 );

# Set number of bins
my $numBin = set_num_bins( \%size2freq_1, \%size2freq_2 );

# Draw a histogram
`two-hists.pl -i $datFile_1 -j $datFile_2 -n $numBin -o $outName`;

################################################### 
# subroutine - read_stats_file
#    IN  = fine name
#    OUT = a hash from 'cluster size' to 'frequency'
###################################################
sub read_stats_file {
    my $file = shift;
    my %size2freq;
    open(IN, $file) or die "Can't open $file to read.";
    my $flg  = 0;
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if( /Cluster size frequency:/ ){ $flg = 1; }
	if( $flg and (@cols==2) ){
	    my ($size, $freq) = @cols;
	    next if $size<2;
	    $size2freq{$size} = $freq;
	}
    }
    close IN;
    return %size2freq;
}


################################################### 
# subroutine - print_data
#    IN  = fine name
#    OUT = reference to a hash from 'cluster size' to 'frequency'
###################################################
sub print_data {
    my $file          = shift;
    my $ref_size2freq = shift;
    open(OUT, ">$file") or die "Can't open $file to write.";
    foreach my $size (sort{$a<=>$b} keys %$ref_size2freq){
	my $freq = $$ref_size2freq{$size};
	next if $size == 1;
	foreach(my $i=0 ; $i<$freq ; ++$i){
	    print OUT $size . "\n";
	}
    }
    close OUT;
}


################################################### 
# subroutine - set_num_bins
#    IN  = reference to a hash from 'cluster size' to 'frequency'
#    IN  = reference to a hash from 'cluster size' to 'frequency'
#    OUT = the number of bins
###################################################
sub set_num_bins {
    my $ref_size2freq_1 = shift;
    my $ref_size2freq_2 = shift;
    my ($min, $max);
    foreach my $size (sort{$a<=>$b} keys %$ref_size2freq_1){
	$min = $size if !defined($min);
	$max = $size if !defined($max);
	$min = $size if $size<$min;
	$max = $size if $size>$max;
    }
    foreach my $size (sort{$a<=>$b} keys %$ref_size2freq_2){
	$min = $size if $size<$min;
	$max = $size if $size>$max;
    }
    return $max-$min+1;
}
