#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_p);
&getopts('hi:j:p:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   organism name 1 (e.g. hsa)
# -j   organism name 2 (e.g. mmu)
# -p   directory containing results of the OASYS and DPD clustering programs

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $org1    = $opt_i or die $usage;
my $org2    = $opt_j or die $usage;
my $dir     = $opt_p or die $usage;
$dir       .= '/' unless $dir =~ /\/$/;
my $o_rbhF  = $dir . $org1 . '.vs.' . $org2 . '.oasys.rbh';
my $b_rbhF  = $dir . $org1 . '.vs.' . $org2 . '.bit.rbh';
my $o_isoF  = $o_rbhF . '.isolated';
my $b_isoF  = $b_rbhF . '.isolated';
my $o_datF  = $dir . $org1 . '.vs.' . $org2 . '.isolated_curve.oasys.dat';
my $b_datF  = $dir . $org1 . '.vs.' . $org2 . '.isolated_curve.bit.dat';
my $gnuFile = $dir . $org1 . '.vs.' . $org2 . '.isolated_curve.gnu';
my $epsFile = $dir . $org1 . '.vs.' . $org2 . '.isolated_curve.eps';
my $sttRate = 0;
my $endRate = 0.1;
my $width   = 0.01;


# Read input files
my %o_rbhL = read_rbh_file( $o_rbhF );
my %o_isoL = read_iso_file( $o_isoF );
my %b_rbhL = read_rbh_file( $b_rbhF );
my %b_isoL = read_iso_file( $b_isoF );

# Output data for plot
print_dat( \%o_rbhL, \%o_isoL, $o_datF );
print_dat( \%b_rbhL, \%b_isoL, $b_datF );

# Make GNUplot file
my $min_x = $sttRate * 100;
my $max_x = $endRate * 100;
open(GNU, ">$gnuFile") or die "Can't open $gnuFile to write.";
print GNU "set terminal postscript color"                   . "\n"
    . "set output \"$epsFile\""                             . "\n"
    . "set xrange [$min_x:$max_x]"                          . "\n"
    . "plot \"$b_datF\" with linespoints linewidth 3, "
    . "\"$o_datF\" with linespoints linewidth 3"            . "\n";
    
close GNU;

# Draw plot
`gnuplot $gnuFile`;


sub read_rbh_file {
    my $file = shift;
    my %list;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split(" ", $_);
	if(@cols==3){
	    my ($id1, $id2, $score) = @cols;
	    $list{$id1."\t".$id2} = $score;
	}
    }
    close IN;
    return %list;
}


sub read_iso_file {
    my $file = shift;
    my %list;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split(" ", $_);
	if(@cols==3){
	    my ($gene1, $gene2, $score) = @cols;
	    my $id1 = gene2id( $gene1 );
	    my $id2 = gene2id( $gene2 );
	    $list{$id1."\t".$id2} = 1;
	}
    }
    close IN;
    return %list;
}


sub gene2id {
    my $gene = shift;
    my @cols = split(":", $gene);
    return $cols[0];
}


sub print_dat {
    my $rbhList = shift;
    my $isoList = shift;
    my $datFile = shift;
    open(DAT, ">$datFile") or die "Can't open $datFile to write.";
    for( my $p=$sttRate ; $p<=$endRate ; $p+=$width ){
	my %curList = rbh_filter( $rbhList, $p );
	my $size    = keys %curList;
	my $isoRate = cal_isolated_rate( \%curList, $isoList );
	print DAT 100*$p .  "\t" . 100*$isoRate . "\n";
    }
    close DAT;
}


sub rbh_filter {
    my $rbhList = shift;
    my $rate    = shift;
    my $size    = keys %$rbhList;
    my $cur_num = $size * (1-$rate);
    my %curList;
    my @list    = reverse sort{$$rbhList{$a}<=>$$rbhList{$b}} keys %$rbhList;
    my $cnt     = 0;
    foreach my $id_pair (@list){
	$curList{$id_pair} = $$rbhList{$id_pair};
	++$cnt;
	last if $cnt > $cur_num;
    }
    return %curList;
}


sub cal_isolated_rate {
    my $curList = shift;
    my $isoList = shift;
    my $num_all = 0;
    my $num_iso = 0;
    foreach my $id_pair ( sort keys %$curList ){
	++$num_all;
	++$num_iso if $$isoList{$id_pair};
    }
    return $num_iso / $num_all;
}
