#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_d $opt_o);
&getopts('hi:j:d:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   organism name 1 (e.g. hsa)
# -j   organism name 2 (e.g. mmu)
# -d   directory containing the RBH pairs
# -o   output directory

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $org_1   = $opt_i or die $usage;
my $org_2   = $opt_j or die $usage;
my $dat_dir = $opt_d or die $usage;
my $out_dir = $opt_o or die $usage;
$dat_dir   .= '/' unless $dat_dir =~ /\/$/;
$out_dir   .= '/' unless $out_dir =~ /\/$/;
( -d $dat_dir ) or die "$dat_dir is not a valid directory.";
( -d $out_dir ) or die "$out_dir is not a valid directory.";

# Get a list of input files
my %wr2file = get_file_list( $dat_dir );
my $numfile = keys %wr2file;
print STDERR $numfile . " files were found." . "\n";

# Draw the plot of the overlapping rate
draw_plot( \%wr2file );


sub get_file_list {
    my $dir = shift;
    my %wr2file;
    opendir(DIR, $dir) or die "Can't open $dir as a directory.";
    while( defined( my $file = readdir(DIR) ) ){
	if( $file=~/^$org_1\.vs\.$org_2\.i_score\.(\d+\.\d+)\.rbh\.cluster.stats$/ ){
	    my $w_ratio = $1;
	    $wr2file{$w_ratio} = $dir.$file;
	}
    }
    closedir DIR;
    return %wr2file;
}


sub draw_plot {
    my $wr2file = shift;
    my $avefile = $out_dir . $org_1 . '.vs.' . $org_2 . '.ave_size.dat';
    my $ortfile = $out_dir . $org_1 . '.vs.' . $org_2 . '.num_ortholog.dat';
    my $clsfile = $out_dir . $org_1 . '.vs.' . $org_2 . '.num_cluster.dat';
    my $isofile = $out_dir . $org_1 . '.vs.' . $org_2 . '.isolated.dat';
    open(AVE, ">$avefile") or die "Can't open $avefile to write.";
    open(ORT, ">$ortfile") or die "Can't open $ortfile to write.";
    open(CLS, ">$clsfile") or die "Can't open $clsfile to write.";
    open(ISO, ">$isofile") or die "Can't open $isofile to write.";
    foreach my $wr ( sort{$a<=>$b} keys %$wr2file ){
	my $file = $$wr2file{$wr};
	my ($ave, $ort, $cls, $iso) = get_cluster_stats( $file );
	print AVE $wr . "\t" . $ave . "\n";
	print ORT $wr . "\t" . $ort . "\n";
	print CLS $wr . "\t" . $cls . "\n";
	print ISO $wr . "\t" . $iso . "\n";
    }
    close AVE; close ORT; close CLS; close ISO;

    plot( $avefile, "ave_size" );
    plot( $ortfile, "num_ortholog" );
    plot( $clsfile, "num_cluster" );
    plot( $isofile, "isolated" );
}

sub plot {
    my $datfile = shift;
    my $name    = shift;
    my $gnufile = $out_dir . $org_1 . '.vs.' . $org_2 . '.' . $name . '.gnu';
    my $epsfile = $out_dir . $org_1 . '.vs.' . $org_2 . '.' . $name . '.eps';
    open(GNU, ">$gnufile") or die "Can't open $gnufile to write.";
    print GNU "set terminal postscript color"                         . "\n"
	. "set output \"" . $epsfile . "\""                           . "\n"
	. "set xtics nomirror"                                        . "\n"
	. "set ytics nomirror"                                        . "\n"
	. "plot \"" . $datfile . "\" with lines "
	. "notitle linewidth 3"                                       . "\n";
    close GNU;
    system "gnuplot $gnufile";
}


sub get_cluster_stats {
    my $file = shift;
    open(IN, $file) or die "Can't open $file to read.";
    my ($ave, $ort, $cls, $iso);
    while(<IN>){
	chomp;
	if( /^Average cluster size\s+= ([\d\.]+)/ ){
	    $ave = $1;
	}
	if( /of orthologs\s+= (\d+)/ ){
	    $ort = $1;
	}
	if( /of clusters\s+= (\d+)/ ){
	    $cls = $1;
	}
	if( /of isolated cluster\s+= (\d+)/ ){
	    $iso = $1;
	}
    }
    close IN;
    return ($ave, $ort, $cls, $iso);
}
