#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_d $opt_o);
&getopts('hi:j:d:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   organism name 1 (e.g. hsa)
# -j   organism name 2 (e.g. mmu)
# -d   directory containing the RBH pairs
# -o   output directory

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $org_1   = $opt_i or die $usage;
my $org_2   = $opt_j or die $usage;
my $dat_dir = $opt_d or die $usage;
my $out_dir = $opt_o or die $usage;
$dat_dir   .= '/' unless $dat_dir =~ /\/$/;
$out_dir   .= '/' unless $out_dir =~ /\/$/;
( -d $dat_dir ) or die "$dat_dir is not a valid directory.";
( -d $out_dir ) or die "$out_dir is not a valid directory.";

# Get a list of input files
my %wr2file = get_file_list( $dat_dir );
my $numfile = keys %wr2file;
print STDERR $numfile . " files were found." . "\n";

# Draw the plot of the overlapping rate
draw_plot( \%wr2file );


sub get_file_list {
    my $dir = shift;
    my %wr2file;
    opendir(DIR, $dir) or die "Can't open $dir as a directory.";
    while( defined( my $file = readdir(DIR) ) ){
	if( $file=~/^$org_1\.vs\.$org_2\.i_score\.(\d+\.\d+)\.rbh\.overlapping_rate/ ){
	    my $w_ratio = $1;
	    $wr2file{$w_ratio} = $dir.$file;
	}
    }
    closedir DIR;
    return %wr2file;
}


sub draw_plot {
    my $wr2file = shift;
    my $datfile_1 = $out_dir . $org_1 . '.vs.' . $org_2 . '.o_rate.1.dat';
    my $datfile_2 = $out_dir . $org_1 . '.vs.' . $org_2 . '.o_rate.2.dat';
    open(DAT1, ">$datfile_1") or die "Can't open $datfile_1 to write.";
    open(DAT2, ">$datfile_2") or die "Can't open $datfile_2 to write.";
    foreach my $wr ( sort{$a<=>$b} keys %$wr2file ){
	my $file = $$wr2file{$wr};
	my ($rate_1, $rate_2) = get_overlapping_rate( $file );
	print DAT1 $wr . "\t" . $rate_1 . "\n";
	print DAT2 $wr . "\t" . $rate_2 . "\n";
    }
    close DAT1;
    close DAT2;

    my $gnufile = $out_dir . $org_1 . '.vs.' . $org_2 . '.o_rate.gnu';
    my $epsfile = $out_dir . $org_1 . '.vs.' . $org_2 . '.o_rate.eps';
    open(GNU, ">$gnufile") or die "Can't open $gnufile to write.";
    print GNU "set terminal postscript color"                         . "\n"
	. "set output \"" . $epsfile . "\""                           . "\n"
	. "set xtics nomirror"                                        . "\n"
	. "set ytics nomirror"                                        . "\n"
	. "plot \"" . $datfile_1 . "\" with lines "
	. "title \"Overlapping rate from the bit score RBHs\" "
	. "linewidth 3, \\"                                           . "\n"
	. "\"" . $datfile_2 . "\" with lines "
	. "title \"Overlapping rate from the oasys score RBHs\" "
	. "linewidth 3"                                               . "\n";
    close GNU;
    system "gnuplot $gnufile";
}


sub get_overlapping_rate {
    my $file = shift;
    open(IN, $file) or die "Can't open $file to read.";
    my ($rate_1, $rate_2);
    while(<IN>){
	chomp;
	if( /^Overlapping rate 1\s+: ([\d\.]+)/ ){
	    $rate_1 = $1;
	}
	if( /^Overlapping rate 2\s+: ([\d\.]+)/ ){
	    $rate_2 = $1;
	}
    }
    close IN;
    return ($rate_1, $rate_2);
}
