#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_d);
&getopts('hi:j:d:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   organism name 1 (e.g. hsa)
# -j   organism name 2 (e.g. mmu)
# -d   directory containing the OASYS results

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $org1    = $opt_i or die $usage;
my $org2    = $opt_j or die $usage;
my $dir     = $opt_d or die $usage;
$dir       .= '/' unless $dir =~ /\/$/;
( -d $dir ) or die "$dir is not a valid directory.";
my $datfile = $dir . $org1 . '.vs.' . $org2 . '.weight_effect.dat';
my $gnufile = $dir . $org1 . '.vs.' . $org2 . '.weight_effect.gnu';
my $epsfile = $dir . $org1 . '.vs.' . $org2 . '.weight_effect.eps';

# Get a list of input files
my %wr2file = get_file_list( $dir );
my $numfile = keys %wr2file;
print STDERR $numfile . " files were found." . "\n";

# Draw the plot of the number of orthologs
make_dat_file( \%wr2file, $datfile );
plot( $datfile, $gnufile, $epsfile );

sub get_file_list {
    my $dir = shift;
    my %wr2file;
    opendir(DIR, $dir) or die "Can't open $dir as a directory.";
    while( defined( my $file = readdir(DIR) ) ){
	if( $file=~/^$org1\.vs\.$org2\.oasys\.rbh\.(\d+\.\d+)$/ ){
	    my $w_ratio = $1;
	    $wr2file{$w_ratio} = $dir.$file;
	}
    }
    closedir DIR;
    return %wr2file;
}


sub make_dat_file {
    my $wr2file = shift;
    my $datfile = shift;
    open(DAT, ">$datfile") or die "Can't open $datfile to write.";
    foreach my $wr ( sort{$a<=>$b} keys %$wr2file ){
	my $file    = $$wr2file{$wr};
	my $n_ortho = get_num_ortholog( $file );
	print DAT $wr . "\t" . $n_ortho . "\n";
    }
    close DAT;
}

sub plot {
    my $datfile = shift;
    my $gnufile = shift;
    my $epsfile = shift;
    open(GNU, ">$gnufile") or die "Can't open $gnufile to write.";
    print GNU "set terminal postscript color"                         . "\n"
	. "set output \"" . $epsfile . "\""                           . "\n"
	. "set xtics nomirror"                                        . "\n"
	. "set ytics nomirror"                                        . "\n"
	. "plot \"" . $datfile . "\" with lines "
	. "notitle linewidth 3"                                       . "\n";
    close GNU;
    system "gnuplot $gnufile";
}


sub get_num_ortholog {
    my $file = shift;
    open(IN, $file) or die "Can't open $file to read.";
    my $n_ortho = 0;
    while(<IN>){
	my @cols = split(" ", $_);
	++$n_ortho if( @cols == 3 )
    }
    close IN;
    return $n_ortho;
}
