#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_p $opt_s $opt_d);
&getopts('hi:j:p:s:d:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   an organism name 1 (e.g. hsa)
# -j   an organism name 2 (e.g. mmu)
# -p   a file containing gene pairs (e.g. *.rbh)
# -s   a directory containing protein sequence files 
# -d   a directory containing dna sequence files

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $org1    = $opt_i or die $usage;
my $org2    = $opt_j or die $usage;
my $paifile = $opt_p or die $usage;
my $pep_dir = $opt_s or die $usage;
my $dna_dir = $opt_d or die $usage;
my $outfile = $paifile . '.dnadist';
$pep_dir   .= '/' unless $pep_dir =~ /\/$/;
$dna_dir   .= '/' unless $dna_dir =~ /\/$/;
my $pep_f1  = $pep_dir . $org1 . '.mfa';
my $pep_f2  = $pep_dir . $org2 . '.mfa';
my $dna_f1  = $dna_dir . $org1 . '.mfa';
my $dna_f2  = $dna_dir . $org2 . '.mfa';
my $tmp_dir = '/tmp/';
my $gapopen = 10.0;
my $gapextend = 0.5;

# Read input files
my %g2pep_1 = read_mfa_file( $pep_f1 );
my $p_num_1 = keys %g2pep_1;
print STDERR $p_num_1 . ' protein sequences were found in ' . $pep_f1 . "\n";
my %g2pep_2 = read_mfa_file( $pep_f2 );
my $p_num_2 = keys %g2pep_2;
print STDERR $p_num_2 . ' protein sequences were found in ' . $pep_f2 . "\n";
my %g2dna_1 = read_mfa_file( $dna_f1 );
my $d_num_1 = keys %g2dna_1;
print STDERR $d_num_1 . ' DNA sequences were found in ' . $dna_f1 . "\n";
my %g2dna_2 = read_mfa_file( $dna_f2 );
my $d_num_2 = keys %g2dna_2;
print STDERR $d_num_2 . ' DNA sequences were found in ' . $dna_f2 . "\n";
my %pairs   = read_pairs_file( $paifile );
my $p_num   = keys %pairs;
print STDERR $p_num . ' gene pairs were found in ' . $paifile . "\n";

open(DIST, ">$outfile") or die "Can't open $outfile to write.";
# Compute global alignment via the needle program
foreach my $pair ( sort keys %pairs ){
    my ($g1, $g2) = split("\t", $pair);
    my ($p_aln1, $p_aln2) 
	= execute_needle( $pair, \%g2pep_1, \%g2pep_2, $tmp_dir );
    my $d_aln1 = pepAln2dnaAln( $p_aln1, $g2dna_1{$g1} );
    my $d_aln2 = pepAln2dnaAln( $p_aln2, $g2dna_2{$g2} );
    my $d_dist = 'N/A';
    if( ($d_aln1 eq 'N/A') or ($d_aln2 eq 'N/A') ){
	print STDERR $g1 . "\n";
    } else { 
	$d_dist = calc_pamdist( $pair, $d_aln1, $d_aln2, $tmp_dir );
	if( $d_dist<0 ){ $d_dist = 'N/A'; }
    }
    print DIST $d_dist . "\n";
    print $d_dist . "\n";
}
close DIST;


sub read_mfa_file {
    my $file = shift;
    my %gid2seq;
    open(MFA, $file) or die "Can't open $file to read.";
    my ($gid,$seq);
    while(<MFA>){
	chomp;
	if(/^>(\S+)/){
	    if(defined $seq){
		$gid2seq{$gid}=$seq;
	    }
	    $gid = $1;
	    undef $seq;
	} else {
	    $_=~s/\s+//g;
	    $seq.=$_;
	}
    }
    if(defined $seq){
	$gid2seq{$gid}=$seq;
    }
    close MFA;
    return %gid2seq;
}


sub read_pairs_file {
    my $file = shift;
    my %pairs;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if( @cols == 3 ){
	    my ($g1, $g2) = @cols[0,1];
	    my @g1 = split(':', $g1); $g1 = $g1[0];
	    my @g2 = split(':', $g2); $g2 = $g2[0];
	    my $pair = join("\t", ($g1, $g2));
	    $pairs{$pair} = 1;
	}
    }
    close IN;
    return %pairs;
}


sub execute_needle {
    my $pair    = shift;
    my $g2seq_1 = shift;
    my $g2seq_2 = shift;
    my $out_dir = shift;
    my ($g1,$g2) = split("\t", $pair);
    my $alnfile = $out_dir . $g1 . '-' . $g2 . '.' . $$ . '.aln';
    print STDERR $alnfile . "\n";
    my $tmp_f1  = $out_dir . $g1 . '.1.' . $$ . '.seq';
    my $tmp_f2  = $out_dir . $g2 . '.2.' . $$ . '.seq';
    print_fasta( $tmp_f1, $g1, $$g2seq_1{$g1} );
    print_fasta( $tmp_f2, $g2, $$g2seq_2{$g2} );
    my $ndl_par 
	= '-asequence ' . $tmp_f1    . ' '
	. '-bsequence ' . $tmp_f2    . ' '
	. '-gapopen '   . $gapopen   . ' '
	. '-gapextend ' . $gapextend . ' '
	. '-outfile '   . $alnfile;
    `needle $ndl_par`;
    `rm $tmp_f1`;
    `rm $tmp_f2`;
    my ($aln1, $aln2) = read_needle_file( $g1, $g2, $alnfile );
    `rm $alnfile`;
    return ($aln1, $aln2);
}


sub print_fasta {
    my $file = shift;
    my $g_id = shift;
    my $seq  = shift;
    open(OUT, ">$file") or die "Can't open $file to write.";
    print OUT '>' . $g_id . "\n"
	. $seq . "\n";
    close OUT;
}


sub read_needle_file {
    my $g1   = shift;
    my $g2   = shift;
    my $file = shift;
    my ($aln1, $aln2);
    open(IN, $file) or die "Can't open $file to read.";
    my $cnt = 0;
    while(<IN>){
	chomp;
	next if( /^\#/ );
	my @cols = split(' ', $_);
	if( @cols == 4 ){
	    next unless( $cols[1] =~ /^\d+$/ );
	    next unless( $cols[3] =~ /^\d+$/ );
	    if( ++$cnt % 2 == 1 ){
		$aln1 .= $cols[2];
	    } else {
		$aln2 .= $cols[2];
	    }
	}
    }
    close IN;
    return ($aln1, $aln2);
}


sub calc_pamdist {
    my $pair = shift;
    my $aln1 = shift; $aln1 =~ s/u/x/ig;
    my $aln2 = shift; $aln2 =~ s/u/x/ig;
    my $out_dir = shift;
    my ($g1, $g2) = split("\t", $pair);
    my $num_seq = 2;
    my $aln_len = length( $aln1 );
    my $pinfile = $out_dir . $g1 . '-' . $g2 . '.' . $$ . '.pam_in';
    my $pamfile = $out_dir . $g1 . '-' . $g2 . '.' . $$ . '.pam';
    if( length($g1) > 10 ){ $g1 = substr($g1, 0, 10); }
    for(my $i=length($g1) ; $i<=10 ; ++$i){ $g1 .= ' '; }
    if( length($g2) > 10 ){ $g2 = substr($g2, 0, 10); }
    for(my $i=length($g2) ; $i<=10 ; ++$i){ $g2 .= ' '; }
    open(OUT, ">$pinfile") or die "Can't open $pinfile to write.";
    print OUT "   " . $num_seq . "   " . $aln_len . "\n"
	. $g1 . $aln1 . "\n"
	. $g2 . $aln2 . "\n";
    close OUT;
    `dnadist $pinfile $pamfile`;
    # `rm $pinfile`;
    my $pam = read_pam_file( $pamfile );
    # `rm $pamfile`;
    return $pam;
}


sub read_pam_file {
    my $file = shift;
    open(IN, $file) or die "Can't open $file to read.";
    my $line = <IN>; chomp $line;
    $line    = <IN>; chomp $line;
    my @cols = split(' ', $line);
    my $pam  = $cols[2];
    return $pam;
}


sub pepAln2dnaAln {
    my $pep_aln = shift;
    my $dna_seq = shift;
    my $dna_aln = "";
    my $seq_idx = 0;
    my $pep_seq = $pep_aln;
    $pep_seq    =~ s/-//g;
    $dna_seq    = match( $dna_seq, $pep_seq );
    if( $dna_seq eq 'N/A' ){ return 'N/A'; }
    for( my $i=0 ; $i<length($pep_aln) ; ++$i ){
	if( substr($pep_aln, $i, 1) eq '-' ){
	    $dna_aln .= '---';
	} else {
	    my $codon = substr( $dna_seq, 3*$seq_idx, 3 );
	    my $aa    = codon2aa( $codon );
	    unless($aa eq substr($pep_aln, $i, 1)){
		print $codon . "\t"
		    . $aa    . "\t" 
		    . substr($pep_aln, $i, 1) . "\n";
		$dna_aln = 'N/A';
		last;
	    }
	    $dna_aln .= $codon;
	    ++$seq_idx;
	}
    }
    return $dna_aln;
}


sub codon2aa {
    my ($codon)=@_;
    die "length of a codon is not appropriate." unless length($codon)==3;
    my %codon2aa;
    $codon2aa{'TTT'}='F';
    $codon2aa{'TTC'}='F';
    $codon2aa{'TTA'}='L';
    $codon2aa{'TTG'}='L';

    $codon2aa{'TCT'}='S';
    $codon2aa{'TCC'}='S';
    $codon2aa{'TCA'}='S';
    $codon2aa{'TCG'}='S';

    $codon2aa{'TAT'}='Y';
    $codon2aa{'TAC'}='Y';
    $codon2aa{'TAA'}='.';
    $codon2aa{'TAG'}='.';

    $codon2aa{'TGT'}='C';
    $codon2aa{'TGC'}='C';
    $codon2aa{'TGA'}='.';
    $codon2aa{'TGG'}='W';

    $codon2aa{'CTT'}='L';
    $codon2aa{'CTC'}='L';
    $codon2aa{'CTA'}='L';
    $codon2aa{'CTG'}='L';

    $codon2aa{'CCT'}='P';
    $codon2aa{'CCC'}='P';
    $codon2aa{'CCA'}='P';
    $codon2aa{'CCG'}='P';

    $codon2aa{'CAT'}='H';
    $codon2aa{'CAC'}='H';
    $codon2aa{'CAA'}='Q';
    $codon2aa{'CAG'}='Q';

    $codon2aa{'CGT'}='R';
    $codon2aa{'CGC'}='R';
    $codon2aa{'CGA'}='R';
    $codon2aa{'CGG'}='R';

    $codon2aa{'ATT'}='I';
    $codon2aa{'ATC'}='I';
    $codon2aa{'ATA'}='I';
    $codon2aa{'ATG'}='M';

    $codon2aa{'ACT'}='T';
    $codon2aa{'ACC'}='T';
    $codon2aa{'ACA'}='T';
    $codon2aa{'ACG'}='T';

    $codon2aa{'AAT'}='N';
    $codon2aa{'AAC'}='N';
    $codon2aa{'AAA'}='K';
    $codon2aa{'AAG'}='K';

    $codon2aa{'AGT'}='S';
    $codon2aa{'AGC'}='S';
    $codon2aa{'AGA'}='R';
    $codon2aa{'AGG'}='R';

    $codon2aa{'GTT'}='V';
    $codon2aa{'GTC'}='V';
    $codon2aa{'GTA'}='V';
    $codon2aa{'GTG'}='V';

    $codon2aa{'GCT'}='A';
    $codon2aa{'GCC'}='A';
    $codon2aa{'GCA'}='A';
    $codon2aa{'GCG'}='A';

    $codon2aa{'GAT'}='D';
    $codon2aa{'GAC'}='D';
    $codon2aa{'GAA'}='E';
    $codon2aa{'GAG'}='E';

    $codon2aa{'GGT'}='G';
    $codon2aa{'GGC'}='G';
    $codon2aa{'GGA'}='G';
    $codon2aa{'GGG'}='G';

    return $codon2aa{$codon} if $codon2aa{$codon};
    die "there is not appropriate codon.";
}


sub match {
    my ($dna,$pep)=@_;
    my $pep_len=length($pep);
    my $dna_len=length($dna);
    my $flgF=0;
    while(-1){
	if($flgF){
	    $dna=substr($dna,1,length($dna)-1);
	    # print length($dna),"\n";
	}
	$flgF=1;

	if(length($dna)<$pep_len*3){
	    #die "Can't match peptide and DNA sequences.";
	    print STDERR "Can't match peptide and DNA sequences.";
	    return 'N/A';
	}
	my $sub_dna=substr($dna,0,$pep_len*3);
	my $sub_pep=dna2pep($sub_dna);
	next if !defined($sub_dna) or !defined($sub_pep);
	if($sub_pep eq $pep){
	    return $sub_dna;
	}
    }
}


sub dna2pep {
    my ($dna)=@_;
    my $pep;
    die "length of DNA sequences is not appropriate." unless length($dna)%3==0;
    my $pep_len=length($dna)/3;
    for(my $i=0;$i<$pep_len;++$i){
	my $codon=substr($dna,$i*3,3);
	my $aa=codon2aa($codon);
	last if $aa eq '.';
	$pep.=$aa;
    }
    return $pep;
}
