#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_p $opt_s $opt_d);
&getopts('hi:j:p:s:d:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   an organism name 1 (e.g. hsa)
# -j   an organism name 2 (e.g. mmu)
# -p   a file containing gene pairs (e.g. *.rbh)
# -s   a directory containing protein sequence files 
# -d   a directory containing dna sequence files

## Others:
# -h   print help

_EOH_
    ;

# Set path
my $scr_dir  = '/Users/hacchy/Documents/slab-works/oasys/oasys_v0_2/scripts/';
my $srs2yn00 = $scr_dir . 'srspair2nuc.pl';
my $parse_yn = $scr_dir . 'parseYn00.pl';

# Get input parameters from command line
die $usage if $opt_h;
my $org1    = $opt_i or die $usage;
my $org2    = $opt_j or die $usage;
my $paifile = $opt_p or die $usage;
my $pep_dir = $opt_s or die $usage;
my $dna_dir = $opt_d or die $usage;
my $outfile = $paifile . '.kaks';
$pep_dir   .= '/' unless $pep_dir =~ /\/$/;
$dna_dir   .= '/' unless $dna_dir =~ /\/$/;
my $pep_f1  = $pep_dir . $org1 . '.mfa';
my $pep_f2  = $pep_dir . $org2 . '.mfa';
my $dna_f1  = $dna_dir . $org1 . '.mfa';
my $dna_f2  = $dna_dir . $org2 . '.mfa';
my $tmp_dir = '/tmp/';
my $gapopen = 10.0;
my $gapextend = 0.5;

# Read input files
my %g2pep_1 = read_mfa_file( $pep_f1 );
my $p_num_1 = keys %g2pep_1;
print STDERR $p_num_1 . ' protein sequences were found in ' . $pep_f1 . "\n";
my %g2pep_2 = read_mfa_file( $pep_f2 );
my $p_num_2 = keys %g2pep_2;
print STDERR $p_num_2 . ' protein sequences were found in ' . $pep_f2 . "\n";
my %g2dna_1 = read_mfa_file( $dna_f1 );
my $d_num_1 = keys %g2dna_1;
print STDERR $d_num_1 . ' DNA sequences were found in ' . $dna_f1 . "\n";
my %g2dna_2 = read_mfa_file( $dna_f2 );
my $d_num_2 = keys %g2dna_2;
print STDERR $d_num_2 . ' DNA sequences were found in ' . $dna_f2 . "\n";
my %pairs   = read_pairs_file( $paifile );
my $p_num   = keys %pairs;
print STDERR $p_num . ' gene pairs were found in ' . $paifile . "\n";

open(KAKS, ">$outfile") or die "Can't open $outfile to write.";
# Compute global alignment via the needle program
foreach my $pair ( sort keys %pairs ){
    my ($g1, $g2) = split("\t", $pair);
    my $srsfile = execute_needle( $pair, \%g2pep_1, \%g2pep_2, $tmp_dir );
    my $dnafile = print_dna_file( $g1, $g2, $g2dna_1{$g1}, $g2dna_2{$g2} );
    my $nucfile = $tmp_dir . $g1 . '-' . $g2 . '.' . $$ . '.nuc';
    my $yn_file = $tmp_dir . $g1 . '-' . $g2 . '.' . $$ . '.yn00';
    my $ctlfile = print_control_file( $g1, $g2, $nucfile, $yn_file, 0 );
    my $sumfile = $tmp_dir . $g1 . '-' . $g2 . '.' . $$ . '.sum';
    my $status  = `$srs2yn00 -i $srsfile -c $dnafile -m 0 -o $nucfile`;
    my $yn00    = 'N/A';
    if( $status =~ /N\/A/ ){
	`rm $srsfile`;
	`rm $dnafile`;
	`rm $nucfile`;
    } else {
	`yn00 $ctlfile`;
	`$parse_yn -i $yn_file -o $sumfile`;
	my %kaks    = read_sum_file( $sumfile );
	`rm $srsfile`;
	`rm $dnafile`;
	`rm $nucfile`;
	#`rm $yn_file`;
	`rm $ctlfile`;
	`rm $sumfile`;
	$yn00 = $kaks{"YN00"};
    }
    if( ($yn00 eq 'N/A') or ($yn00 eq 'Inf') ){ print KAKS 'N/A' . "\n"; }
    else { print KAKS $yn00 . "\n"; print $yn00 . "\n"; }
}
close KAKS;


sub read_mfa_file {
    my $file = shift;
    my %gid2seq;
    open(MFA, $file) or die "Can't open $file to read.";
    my ($gid,$seq);
    while(<MFA>){
	chomp;
	if(/^>(\S+)/){
	    if(defined $seq){
		$gid2seq{$gid}=$seq;
	    }
	    $gid = $1;
	    undef $seq;
	} else {
	    $_=~s/\s+//g;
	    $seq.=$_;
	}
    }
    if(defined $seq){
	$gid2seq{$gid}=$seq;
    }
    close MFA;
    return %gid2seq;
}


sub read_pairs_file {
    my $file = shift;
    my %pairs;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if( @cols == 3 ){
	    my ($g1, $g2) = @cols[0,1];
	    my @g1 = split(':', $g1); $g1 = $g1[0];
	    my @g2 = split(':', $g2); $g2 = $g2[0];
	    my $pair = join("\t", ($g1, $g2));
	    $pairs{$pair} = 1;
	}
    }
    close IN;
    return %pairs;
}


sub execute_needle {
    my $pair    = shift;
    my $g2seq_1 = shift;
    my $g2seq_2 = shift;
    my $out_dir = shift;
    my ($g1,$g2) = split("\t", $pair);
    my $alnfile = $out_dir . $g1 . '-' . $g2 . '.' . $$ . '.aln';
    print STDERR $alnfile . "\n";
    my $tmp_f1  = $out_dir . $g1 . '.1.' . $$ . '.seq';
    my $tmp_f2  = $out_dir . $g2 . '.2.' . $$ . '.seq';
    print_fasta( $tmp_f1, $g1, $$g2seq_1{$g1} );
    print_fasta( $tmp_f2, $g2, $$g2seq_2{$g2} );
    my $ndl_par 
	= '-asequence ' . $tmp_f1    . ' '
	. '-bsequence ' . $tmp_f2    . ' '
	. '-gapopen '   . $gapopen   . ' '
	. '-gapextend ' . $gapextend . ' '
	. '-outfile '   . $alnfile;
    `needle $ndl_par`;
    `rm $tmp_f1`;
    `rm $tmp_f2`;
    return $alnfile;
}


sub print_fasta {
    my $file = shift;
    my $g_id = shift;
    my $seq  = shift;
    open(OUT, ">$file") or die "Can't open $file to write.";
    print OUT '>' . $g_id . "\n"
	. $seq . "\n";
    close OUT;
}


sub print_dna_file {
    my $g1      = shift;
    my $g2      = shift;
    my $dna1    = shift;
    my $dna2    = shift;
    my $outfile = $tmp_dir . $g1 . '-' . $g2 . '.' . $$ . '.dna';
    open(OUT, ">$outfile") or die "Can't open $outfile to write.";
    print OUT ">$g1" . "\n"
	. $dna1      . "\n"
	. ">$g2"     . "\n"
	. $dna2      . "\n";
    close OUT;
    return $outfile;
}


sub print_control_file {
    my $g1      = shift;
    my $g2      = shift;
    my $nucfile = shift;
    my $yn_file = shift;
    my $flg_m   = shift;
    my $ctlfile = $tmp_dir . $g1 . '-' . $g2 . '.' . $$ . '.ctl';
    open(CTL, ">$ctlfile") or die "Can't open $ctlfile to write.";
    print CTL ''
	. 'seqfile    = ' . $nucfile . "\n"
	. 'outfile    = ' . $yn_file . "\n"
	. 'verbose    = ' . 0        . "\n"
	. 'icode      = ' . $flg_m   . "\n"
	. 'weighting  = ' . 0        . "\n"
	. 'commonf3x4 = ' . 0        . "\n"
	. 'ndata      = ' . 1        . "\n";
    close CTL;
    return $ctlfile;
}


sub read_sum_file {
    my $file = shift;
    open(IN, $file) or die "Can't open $file to read.";
    my %kaks;
    while(<IN>){
	chomp;
	my @cols = split(" ", $_);
	if( @cols == 5 ){
	    my ($method, $val) = @cols[0,2];
	    $kaks{$method} = $val;
	}
    }
    close IN;
    return %kaks;
}


