#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_p $opt_s);
&getopts('hi:j:p:s:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   an organism name 1 (e.g. hsa)
# -j   an organism name 2 (e.g. mmu)
# -p   a file containing gene pairs (e.g. *.rbh)
# -s   a directory containing mfa files 

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $org1    = $opt_i or die $usage;
my $org2    = $opt_j or die $usage;
my $paifile = $opt_p or die $usage;
my $mfa_dir = $opt_s or die $usage;
my $outfile = $paifile . '.pamdist';
$mfa_dir   .= '/' unless $mfa_dir =~ /\/$/;
my $mfa_f1  = $mfa_dir . $org1 . '.mfa';
my $mfa_f2  = $mfa_dir . $org2 . '.mfa';
my $tmp_dir = '/tmp/';
my $gapopen = 10.0;
my $gapextend = 0.5;

# Read input files
my %g2seq_1 = read_mfa_file( $mfa_f1 );
my $g_num_1 = keys %g2seq_1;
print STDERR $g_num_1 . ' genes were found in ' . $mfa_f1 . "\n";
my %g2seq_2 = read_mfa_file( $mfa_f2 );
my $g_num_2 = keys %g2seq_2;
print STDERR $g_num_2 . ' genes were found in ' . $mfa_f2 . "\n";
my %pairs   = read_pairs_file( $paifile );
my $p_num   = keys %pairs;
print STDERR $p_num . ' gene pairs were found in ' . $paifile . "\n";

open(PAMDIST, ">$outfile") or die "Can't open $outfile to write.";
# Compute global alignment via the needle program
foreach my $pair ( sort keys %pairs ){
    my ($aln1, $aln2) 
	= execute_needle( $pair, \%g2seq_1, \%g2seq_2, $tmp_dir );
    my $pam = calc_pamdist( $pair, $aln1, $aln2, $tmp_dir );
    # print PAMDIST $pair . "\t" . $pam . "\n";
    print PAMDIST $pam . "\n";
}
close PAMDIST;


sub read_mfa_file {
    my $file = shift;
    my %gid2seq;
    open(MFA, $file) or die "Can't open $file to read.";
    my ($gid,$seq);
    while(<MFA>){
	chomp;
	if(/^>(\S+)/){
	    if(defined $seq){
		$gid2seq{$gid}=$seq;
	    }
	    $gid = $1;
	    undef $seq;
	} else {
	    $_=~s/\s+//g;
	    $seq.=$_;
	}
    }
    if(defined $seq){
	$gid2seq{$gid}=$seq;
    }
    close MFA;
    return %gid2seq;
}


sub read_pairs_file {
    my $file = shift;
    my %pairs;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if( @cols == 3 ){
	    my ($g1, $g2) = @cols[0,1];
	    my @g1 = split(':', $g1); $g1 = $g1[0];
	    my @g2 = split(':', $g2); $g2 = $g2[0];
	    my $pair = join("\t", ($g1, $g2));
	    $pairs{$pair} = 1;
	}
    }
    close IN;
    return %pairs;
}


sub execute_needle {
    my $pair    = shift;
    my $g2seq_1 = shift;
    my $g2seq_2 = shift;
    my $out_dir = shift;
    my ($g1,$g2) = split("\t", $pair);
    my $alnfile = $out_dir . $g1 . '-' . $g2 . '.' . $$ . '.aln';
    print STDERR $alnfile . "\n";
    my $tmp_f1  = $out_dir . $g1 . '.1.' . $$ . '.seq';
    my $tmp_f2  = $out_dir . $g2 . '.2.' . $$ . '.seq';
    print_fasta( $tmp_f1, $g1, $g2seq_1{$g1} );
    print_fasta( $tmp_f2, $g2, $g2seq_2{$g2} );
    my $ndl_par 
	= '-asequence ' . $tmp_f1    . ' '
	. '-bsequence ' . $tmp_f2    . ' '
	. '-gapopen '   . $gapopen   . ' '
	. '-gapextend ' . $gapextend . ' '
	. '-outfile '   . $alnfile;
    `needle $ndl_par`;
    `rm $tmp_f1`;
    `rm $tmp_f2`;
    my ($aln1, $aln2) = read_needle_file( $g1, $g2, $alnfile );
    `rm $alnfile`;
    return ($aln1, $aln2);
}


sub print_fasta {
    my $file = shift;
    my $g_id = shift;
    my $seq  = shift;
    open(OUT, ">$file") or die "Can't open $file to write.";
    print OUT '>' . $g_id . "\n"
	. $seq . "\n";
    close OUT;
}


sub read_needle_file {
    my $g1   = shift;
    my $g2   = shift;
    my $file = shift;
    my ($aln1, $aln2);
    open(IN, $file) or die "Can't open $file to read.";
    my $cnt = 0;
    while(<IN>){
	chomp;
	next if( /^\#/ );
	my @cols = split(' ', $_);
	if( @cols == 4 ){
	    next unless( $cols[1] =~ /^\d+$/ );
	    next unless( $cols[3] =~ /^\d+$/ );
	    if( ++$cnt % 2 == 1 ){
		$aln1 .= $cols[2];
	    } else {
		$aln2 .= $cols[2];
	    }
	}
    }
    close IN;
    return ($aln1, $aln2);
}


sub calc_pamdist {
    my $pair = shift;
    my $aln1 = shift; $aln1 =~ s/u/x/ig;
    my $aln2 = shift; $aln2 =~ s/u/x/ig;
    my $out_dir = shift;
    my ($g1, $g2) = split("\t", $pair);
    my $num_seq = 2;
    my $aln_len = length( $aln1 );
    my $pinfile = $out_dir . $g1 . '-' . $g2 . '.' . $$ . '.pam_in';
    my $pamfile = $out_dir . $g1 . '-' . $g2 . '.' . $$ . '.pam';
    if( length($g1) > 10 ){ $g1 = substr($g1, 0, 10); }
    for(my $i=length($g1) ; $i<=10 ; ++$i){ $g1 .= ' '; }
    if( length($g2) > 10 ){ $g2 = substr($g2, 0, 10); }
    for(my $i=length($g2) ; $i<=10 ; ++$i){ $g2 .= ' '; }
    open(OUT, ">$pinfile") or die "Can't open $pinfile to write.";
    print OUT "   " . $num_seq . "   " . $aln_len . "\n"
	. $g1 . $aln1 . "\n"
	. $g2 . $aln2 . "\n";
    close OUT;
    `protdist $pinfile $pamfile`;
    `rm $pinfile`;
    my $pam = read_pam_file( $pamfile );
    `rm $pamfile`;
    print $pam . "\n";
    return $pam;
}


sub read_pam_file {
    my $file = shift;
    open(IN, $file) or die "Can't open $file to read.";
    my $line = <IN>; chomp $line;
    $line    = <IN>; chomp $line;
    my @cols = split(' ', $line);
    my $pam  = $cols[2];
    return $pam;
}
