#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);
use DB_File;

use vars qw ($opt_h $opt_i $opt_d $opt_b $opt_o $opt_B $opt_L);
&getopts('hi:d:b:o:B:L:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a directory for organism A (used as query)
# -d   a directory for organism B (used as database)
# -b   a file containing blast results (-m 9 format, *.blastp)
# -o   an oasys_in file for output (*.oasys_in)

## Optional:
# -B   bit score threshold to detect ortholog candidates (default = 50bits)
# -L   length threshold to detect ortholog candidates (default = 50%)

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $q_dir     = $opt_i or die $usage;
my $d_dir     = $opt_d or die $usage;
my $blsFile   = $opt_b or die $usage;
my $outFile   = $opt_o or die $usage;
my $cut_bit   = 50;
my $cut_len   = 50;
$cut_bit      = $opt_B if defined $opt_B;
$cut_len      = $opt_L if defined $opt_L;
$q_dir       .= '/' unless $q_dir =~ /\/$/;
$d_dir       .= '/' unless $d_dir =~ /\/$/;

# Set file names
my $dbmGeneFile_q        = $q_dir . 'gene.dbm';
my $dbmGeneFile_d        = $d_dir . 'gene.dbm';
my $dbmGene2pepLenFile_q = $q_dir . 'gene2pepLen.dbm';
my $dbmGene2pepLenFile_d = $d_dir . 'gene2pepLen.dbm';
(-f $dbmGeneFile_q) or die "Doesn't exist $dbmGeneFile_q";
(-f $dbmGeneFile_d) or die "Doesn't exist $dbmGeneFile_d";
(-f $dbmGene2pepLenFile_q) or die "Doesn't exist $dbmGene2pepLenFile_q";
(-f $dbmGene2pepLenFile_d) or die "Doesn't exist $dbmGene2pepLenFile_d";

# Open DBM files
my (%gene_id2name_q, %gene_id2name_d, %gene2pepLen_q, %gene2pepLen_d);
dbmopen %gene_id2name_q, $dbmGeneFile_q, 0666
    or die "Can't open $dbmGeneFile_q as a DBM file:$!";
dbmopen %gene_id2name_d, $dbmGeneFile_d, 0666
    or die "Can't open $dbmGeneFile_d as a DBM file:$!";
dbmopen %gene2pepLen_q, $dbmGene2pepLenFile_q, 0666
    or die "Can't open $dbmGene2pepLenFile_q as a DBM file:$!";
dbmopen %gene2pepLen_d, $dbmGene2pepLenFile_d, 0666
    or die "Can't open $dbmGene2pepLenFile_d as a DBM file:$!";

# Get hashes from 'gene name' to 'peptide length'
my %gene_name2pepLen_q = get_name2pepLen(\%gene_id2name_q, \%gene2pepLen_q);
my %gene_name2pepLen_d = get_name2pepLen(\%gene_id2name_d, \%gene2pepLen_d);

# Close DBM files
dbmclose %gene_id2name_q;
dbmclose %gene_id2name_d;
dbmclose %gene2pepLen_q;
dbmclose %gene2pepLen_d;

# Parse blast results
my %pair2bit  = read_blast_file( $blsFile, \%gene_name2pepLen_q, 
				 \%gene_name2pepLen_d, $cut_bit, $cut_len );
my $num_hits  = keys %pair2bit;
print STDERR $num_hits . " hits in $blsFile" . "\n";

# Print the input file of the OASYS program
print_homologs( \%pair2bit, $outFile );


sub get_name2pepLen {
    my $ref_id2name     = shift;
    my $ref_gene2pepLen = shift;
    my %name2pepLen;
    foreach my $id (sort keys %$ref_id2name){
	my $name = $$ref_id2name{$id};
	my $pepLen = $$ref_gene2pepLen{$id};
	$name2pepLen{$name} = $pepLen;
    }
    return %name2pepLen;
}


sub read_blast_file {
    my $file     = shift;
    my $id2len_1 = shift;
    my $id2len_2 = shift;
    my $cut_bit  = shift;
    my $cut_len  = shift;
    my %pair2bit;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	next if /^#/;
	my @cols = split(" ", $_);
	if( @cols == 12 ){
	    my ($query, $subject, $len, $bit) = @cols[0, 1, 3, 11];
	    if( $bit >= $cut_bit ){
		if( $len >= $$id2len_1{$query}*$cut_len/100 ){
		    if( $len >= $$id2len_2{$subject}*$cut_len/100 ){
			my $pair = make_pair($query, $subject);
			$pair2bit{$pair} = $bit;
		    }
		}
	    }
	}
    }
    close IN;
    return %pair2bit;
}


sub print_homologs {
    my $pair2bit = shift;
    my $outFile  = shift;
    open(OUT, ">$outFile") or die "Can't open $outFile to write.";
    foreach my $pair ( sort keys %$pair2bit ){
	my ($id_1, $id_2) = get_pair_id( $pair );
	my $bit   = $$pair2bit{$pair};
	print OUT    $id_1 . "\t" . $id_2 . "\t" . $bit . "\n";
    }
    close OUT;
}


sub make_pair {
    my $val1 = shift;
    my $val2 = shift;
    return $val1 . ' ' . $val2;
}


sub get_pair_id {
    my $pair = shift;
    return split(" ", $pair);
}
