#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_d);
&getopts('hi:j:d:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   organism name 1 (e.g. hsa)
# -j   organism name 2 (e.g. mmu)
# -d   directory containing files output by MSOAR

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $org1    = $opt_i or die $usage;
my $org2    = $opt_j or die $usage;
my $dir     = $opt_d or die $usage;
$dir       .= '/' unless $dir =~ /\/$/;
my $in_file = $dir . $org1 . '.vs.' . $org2 . '.msoar';
my $pairOut = $dir . $org1 . '.vs.' . $org2 . '.msoar.pairs';
my $grouOut = $dir . $org1 . '.vs.' . $org2 . '.msoar.groups';

# Read input files
my (%gene1, %gene2);
my %pairs     = read_ens_comp_file( \%gene1, \%gene2, $in_file );
my %groups    = get_groups( \%pairs, \%gene1, \%gene2 );
my $num_gene1 = keys %gene1;
my $num_gene2 = keys %gene2;
my $num_ortho = keys %pairs;
my $num_group = keys %groups;
print STDERR $num_gene1 . " unique genes in organism A" . "\n";
print STDERR $num_gene2 . " unique genes in organism B" . "\n";
print STDERR $num_ortho . " ortholog pairs" . "\n";
print STDERR $num_group . " ortholog groups" . "\n";

# Print results
print_pairs( \%pairs, $pairOut );
print_groups( \%groups, $grouOut );

sub read_ens_comp_file {
    my $gene1 = shift;
    my $gene2 = shift;
    my $file  = shift;
    my %pairs;
    open(IN, $file) or die "Can't open $file to read.";
    my $head  = <IN>;
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if(@cols==2){
	    my ($g1, $g2) = @cols;
	    $pairs{$_}    = 1;
	    $$gene1{$g1}  = 1;
	    $$gene2{$g2}  = 1;
	}
    }
    close IN;
    return %pairs;
}


sub get_groups {
    my $pairs = shift;
    my $gene1 = shift;
    my $gene2 = shift;
    my %groups;
    my (%g1_gid, %g2_gid, %gid_g1list, %gid_g2list);
    my $next_gid = 1;
    foreach my $pair (sort keys %$pairs){
	my ($g1, $g2) = split(" ", $pair);
	if( !$g1_gid{$g1} and !$g2_gid{$g2} ){
	    $g1_gid{$g1} = $next_gid;
	    $g2_gid{$g2} = $next_gid;
	    ++$next_gid;
	    my @list1    = ($g1);
	    my @list2    = ($g2);
	    $gid_g1list{$next_gid} = \@list1;
	    $gid_g2list{$next_gid} = \@list2;
	} elsif ( $g1_gid{$g1} ){
	    my $gid      = $g1_gid{$g1};
	    $g2_gid{$g2} = $gid;
	    if( $gid_g2list{$gid} ){
		my $list2 = $gid_g2list{$gid};
		push(@$list2, $g2);
	    } else {
		my @list2 = ($g2);
		$gid_g2list{$gid} = \@list2;
	    }
	} elsif ( $g2_gid{$g2} ){
	    my $gid      = $g2_gid{$g2};
	    $g1_gid{$g1} = $gid;
	    if( $gid_g1list{$gid} ){
		my $list1 = $gid_g1list{$gid};
		push(@$list1, $g1);
	    } else {
		my @list1 = ($g1);
		$gid_g1list{$gid} = \@list1;
	    }
	} else {
	    if( $g1_gid{$g1} != $g2_gid{$g2} ){
		merge_groups( \%g1_gid, \%g2_gid, \%gid_g1list, \%gid_g2list, 
			      $g1_gid{$g1}, $g2_gid{$g2}, $next_gid );
		++$next_gid;
	    }
	}
    }
    my $oid = 1;
    foreach my $gid (sort{$a<=>$b} keys %gid_g1list){
	my $g1list = $gid_g1list{$gid};
	my $g2list = $gid_g2list{$gid};
	my @list   = ($g1list, $g2list);
	$groups{$oid++} = \@list;
    }
    return %groups;
}


sub merge_groups {
    my $g1_gid     = shift;
    my $g2_gid     = shift;
    my $gid_g1list = shift;
    my $gid_g2list = shift;
    my $id1        = shift;
    my $id2        = shift;
    my $next_gid   = shift;
    foreach my $g1 (sort keys %$g1_gid){
	if( $$g1_gid{$g1}==$id1 or $$g1_gid{$g1}==$id2 ){
	    $$g1_gid{$g1} = $next_gid;
	}
    }
    foreach my $g2 ( sort keys %$g2_gid ){
	if( $$g2_gid{$g2}==$id1 or $$g2_gid{$g2}==$id2 ){
	    $$g2_gid{$g2} = $next_gid;
	}
    }
    my $list11 = $$gid_g1list{$id1};
    my $list12 = $$gid_g1list{$id2};
    my $list21 = $$gid_g2list{$id1};
    my $list22 = $$gid_g2list{$id2};
    my @list1  = (@$list11, @$list12);
    my @list2  = (@$list21, @$list22);
    $$gid_g1list{$id1} = undef;
    $$gid_g1list{$id2} = undef;
    $$gid_g2list{$id1} = undef;
    $$gid_g2list{$id2} = undef;
    $$gid_g1list{$next_gid} = \@list1;
    $$gid_g2list{$next_gid} = \@list2;
}


sub print_pairs {
    my $ortho = shift;
    my $file  = shift;
    open(OUT, ">$file") or die "Can't open $file to write.";
    foreach ( sort keys %$ortho ){
	print OUT $_ . "\t" . '0' . "\n";
    }
    close OUT;
}


sub print_groups {
    my $groups     = shift;
    my $file       = shift;
    open(OUT, ">$file") or die "Can't open $file to write.";
    foreach my $gid ( sort{$a<=>$b} keys %$groups ){
	my $list   = $$groups{$gid};
	my ($g1list, $g2list) = @$list;
	print OUT 'Group ID ' . $gid . "\n"
	    . "\t" . join(' ', @$g1list) . "\n"
	    . "\t" . join(' ', @$g2list) . "\n";
    }
    close OUT;
}
