#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_n $opt_o);
&getopts('hi:j:n:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing a list of scalar values
# -j   a file containing a list of scalar values
# -n   number of bins
# -o   job name

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $dfile_1 = $opt_i or die $usage;
my $dfile_2 = $opt_j or die $usage;
my $num_bin = $opt_n or die $usage;
my $jobname = $opt_o or die $usage;
my $datfile = $jobname . '.dat';
my $gnufile = $jobname . '.gnu';
my $epsfile = $jobname . '.eps';

# Read input files
my @val1    = read_data_file( $dfile_1 );
my @val2    = read_data_file( $dfile_2 );

# Draw a multi-histogram
my @data    = (\@val1, \@val2);
my %breaks  = set_breaks( \@data, $num_bin );
my %hist_1  = set_histogram( \@val1, \%breaks );
my %hist_2  = set_histogram( \@val2, \%breaks );
my @histos  = (\%hist_1, \%hist_2);
make_data_file( \%breaks, \@histos, $datfile );
my $num_dat = @data;
my $interval= set_interval( \%breaks );
make_gnu_file( $datfile, $gnufile, $epsfile, $interval );
system "gnuplot $gnufile";


sub read_data_file {
    my $file = shift;
    my @vals;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if( @cols == 1 ){
	    my $val = $cols[0];
	    push( @vals, $val );
	}
    }
    close IN;
    return @vals;
}


sub set_breaks {
    my $data    = shift;
    my $num_bin = shift;
    my %breaks;
    my ($min, $max);
    foreach my $vals ( @$data ){
	foreach my $val ( sort{$a<=>$b} @$vals ){
	    $min = $val if( !defined($min) or $val<$min );
	    $max = $val if( !defined($max) or $val>$max );
	}
    }
    my $interval = ($max-$min+1)/$num_bin;
    for( my $i=0 ; $i<$num_bin ; ++$i ){
	my $break   = $min + $interval*$i;
	$breaks{$i} = $break;
    }
    return %breaks;
}


sub set_histogram {
    my $vals    = shift;
    my $breaks  = shift;
    my %hist;
    my $num_val = @$vals;
    my $num_bin = keys %$breaks;
    foreach my $val (@$vals){
	my $flg_cnt = 0;
	foreach my $bin (sort{$a<=>$b} keys %$breaks){
	    my $break = $$breaks{$bin};
	    if( $break > $val ){
		++$hist{--$bin};
		$flg_cnt = 1;
		last;
	    }
	}
	++$hist{$num_bin-1} unless $flg_cnt;
    }
    foreach my $bin (sort{$a<=>$b} keys %$breaks){
	$hist{$bin} = 0 if !defined( $hist{$bin} );
	print $bin . "\t" . $hist{$bin} . "\t";
# 	if( $hist{$bin} ){ $hist{$bin} /= $num_val; }
# 	else             { $hist{$bin}  = 0; }
	print $hist{$bin} . "\n";
    }
    print "\n";
    return %hist;
}


sub make_data_file {
    my $breaks  = shift;
    my $histos  = shift;
    my $datfile = shift;
    open(DAT, ">$datfile") or die "Can't open $datfile to write.";
    foreach my $bin (sort{$a<=>$b} keys %$breaks){
	my $break = $$breaks{$bin};
	my @freqs = get_frequencies( $histos, $bin );
	print DAT $break . "\t" . join("\t", @freqs) . "\n";
    }
    close DAT;
}


sub set_interval {
    my $ref_breaks = shift;
    my $interval   = $$ref_breaks{1} - $$ref_breaks{0};
    return $interval;
}


sub get_frequencies {
    my $histos = shift;
    my $bin    = shift;
    my @freqs;
    foreach my $hist (@$histos){
	my $freq = $$hist{$bin};
	push(@freqs, $freq);
    }
    return @freqs;
}


sub make_gnu_file {
    my $datfile = shift;
    my $gnufile = shift;
    my $epsfile = shift;
    my $interval = shift;
    open(GNU, ">$gnufile") or die "Can't open $gnufile to write.";
    print GNU 'set terminal postscript'                                 . "\n"
	. 'set output "' . $epsfile . '"'                               . "\n"
	. 'set xrange [0:]'                                             . "\n"
	. 'plot "' . $datfile . '" using 1:2:(' . $interval/3 . ') '
	. 'with boxes fs pattern 3 ti "1", "'
	. $datfile . '" using ($1+' . ($interval/3+$interval/10) . '):3:('
	. $interval/3 . ') with boxes fs pattern 2 ti "2"'
	. "\n";
    close GNU;
}
