// **************************************************************************
//  agg.cpp
// **************************************************************************

#include "agg.h"

// **************************************************************************
// Class param_agg
// Initialize parameter values
param_agg::param_agg( vector<double>& x ){
  pdf_agg agg;
  agg.set_param(this);
  agg.fit_ml(x);
  cerr << "log L  = " << agg.log_L( x )  << endl
       << "log dL = " << agg.log_dL( x ) << endl;
}


// **************************************************************************
// Class param_agg
// Initialize parameter values
param_agg::param_agg( vector<double>& x, vector<double>& w ){
  pdf_agg agg;
  agg.set_param(this);
  agg.fit_ml(x, w);
  cerr << "log L  = " << agg.log_L( x, w )  << endl
       << "log dL = " << agg.log_dL( x, w ) << endl;
}


// **************************************************************************
// Class param_agg
// Show parameter values
void param_agg::show(){
  cerr << "Location   = " << loc      << endl
       << "SD (left)  = " << sd_left  << endl
       << "SD (right) = " << sd_right << endl
       << "Shape      = " << shape    << endl;
}


// **************************************************************************
// Class pdf_agg
// Fit the observed data to this PDF
void pdf_agg::fit_ml( vector<double>& x ){
  p->loc      = cal_mode_hist( x );
  p->sd_left  = cal_sd_left( x );
  if(p->sd_left < MIN_SD_AGG) p->sd_left = MIN_SD_AGG;
  p->sd_right = cal_sd_right( x );
  if(p->sd_right < MIN_SD_AGG) p->sd_right = MIN_SD_AGG;  
  p->shape    = cal_shape( x );
}


// **************************************************************************
// Class pdf_agg
// Fit the observed data to this PDF
void pdf_agg::fit_ml( vector<double>& x, vector<double>& w ){
  p->loc      = cal_mode_hist( x, w );
  p->sd_left  = cal_sd_left( x, w );
  if(p->sd_left < MIN_SD_AGG) p->sd_left = MIN_SD_AGG;
  p->sd_right = cal_sd_right( x, w );
  if(p->sd_right < MIN_SD_AGG) p->sd_right = MIN_SD_AGG;
  p->shape    = cal_shape( x, w );
}


// **************************************************************************
// Class pdf_agg
// Draw this PDF via GNUplot
void pdf_agg::plot( string name ){
  string gnufile   = name + ".agg.gnu";
  string epsfile   = name + ".agg.eps";
  string pd_file = name + ".agg.plot.dat";
  double x_min   = p->loc - 3 * p->sd_left;
  double x_max   = p->loc + 3 * p->sd_right;
  print_sample_points( pd_file, x_min, x_max );
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript"                                      << endl
      << "set output \"" << epsfile << "\""                             << endl
      << "set xrange [" << x_min << ":" << x_max << "]"                 << endl
      << "plot \"" << pd_file << "\" with lines "
      << "title \"AGG PDF "
      << "(location=" << p->loc << ", sd left=" << p->sd_left
      << ", sd right=" << p->sd_right 
      << ", shape=" << p->shape << ")\" linewidth 2" 
      << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
}


// **************************************************************************
// Class pdf_agg
// Draw this PDF with a histogram via GNUplot
void pdf_agg::plot( string name, vector<double>& x, size_t num_bin ){
  histogram h( num_bin, x );
  plot( name, h );
}


// **************************************************************************
// Class pdf_agg
// Draw this PDF with a histogram via GNUplot
void pdf_agg::plot( string name, vector<double>& x, 
		    vector<double>& w, size_t num_bin ) {
  histogram h( num_bin, x, w );
  plot( name, h );
}


// **************************************************************************
// Class pdf_agg
// Draw this PDF with a histogram via GNUplot
void pdf_agg::plot( string name, histogram& h ){
  string datfile   = name + ".agg.hist.dat";
  string pd_file   = name + ".agg.plot.dat";
  string gnufile   = name + ".agg.gnu";
  string epsfile   = name + ".agg.eps";
  double x_min     = h.get_min();
  double x_max     = h.get_max();
  double y1max     = h.get_max_val()*1.1;
  double y2max;
  if( p->loc >= x_max ){
    y2max = eval(p->loc)*y1max/h.get(h.size()-1);
  } else {
    y2max = eval(p->loc)*y1max/h.get_val(p->loc);
  }
  h.print( datfile );
  print_sample_points( pd_file, x_min, x_max );
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript color"                                << endl
      << "set output \"" << epsfile << "\""                             << endl
      << "set xrange  [" << x_min << ":" << x_max << "]"                << endl
      << "set yrange  [" << 0     << ":" << y1max << "]"                << endl
      << "set y2range [" << 0     << ":" << y2max << "]"                << endl
      << "set xtics nomirror"                                           << endl
      << "set ytics nomirror"                                           << endl
      << "set style line 1 lt 1"                                        << endl
      << "set style line 2 lt 2"                                        << endl
      << "set style fill solid 0.7"                                     << endl
      << "plot \"" << datfile << "\" with boxes axis x1y1 "
      << "title \"Histogram of the observed data\" lt 2, "
      << "\"" << pd_file << "\" with lines axis x1y2 "
      << "title \"AGG PDF "
      << "(location=" << p->loc 
      << ", sd left=" << p->sd_left
      << ", sd right=" << p->sd_right 
      << ", shape=" << p->shape << ")\" linewidth 3 lt 1"
      << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
}


// **************************************************************************
// Class pdf_agg
// Draw this PDF with various sets of parameters
void pdf_agg::plot( string name, vector<void*>& param_list ){
  string gnufile = name + ".agg.gnu";
  string epsfile = name + ".agg.eps";
  param_agg* cur_param = p;
  double x_min = p->loc - 3*p->sd_left;
  double x_max = p->loc + 3*p->sd_right;
  vector<string> pd_list;
  vector<double> loc_list, sdl_list, sdr_list, shape_list;
  for( unsigned i=0 ; i<param_list.size() ; ++i ){
    string pd_file = name + ".agg.plot." + int2string(i) + ".dat";
    p = (param_agg*)param_list[i];
    loc_list.push_back( p->loc );
    sdl_list.push_back( p->sd_left );
    sdr_list.push_back( p->sd_right );
    shape_list.push_back( p->shape );
    print_sample_points( pd_file, x_min, x_max );
    pd_list.push_back( pd_file );
  }
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript"                                      << endl
      << "set output \"" << epsfile << "\""                             << endl
      << "set xrange [" << x_min << ":" << x_max << "]"                 << endl
      << "set xtics nomirror"                                           << endl
      << "set ytics nomirror"                                           << endl
      << "plot \"" << pd_list[0] << "\" with lines "
      << "title \"loc=" << loc_list[0] 
      << ",sd(left)="   << sdl_list[0]
      << ",sd(right)="  << sdr_list[0]
      << ",shape="      << shape_list[0] << "\" linewidth 2";
  for( unsigned i=1 ; i<pd_list.size() ; ++i ){
    ofs << ", \"" << pd_list[i] << "\" with lines "
	<< "title \"loc=" << loc_list[i] 
	<< ",sd(left)="   << sdl_list[i]
	<< ",sd(right)="  << sdr_list[i]
	<< ",shape="      << shape_list[i] << "\" linewidth 2";
  }
  ofs << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
  p = cur_param;
}


// **************************************************************************
// Class pdf_agg
// Evaluate this PDF
double pdf_agg::eval( double x ){
  double gamma_a = 1.0/(p->sd_left+p->sd_right)*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double A = p->shape*gamma_a/gsl_sf_gamma(1.0/p->shape);
  if( x <= p->loc ){
    double gamma_l = 1.0/p->sd_left*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
    return A * exp( -1.0 * pow(gamma_l, p->shape) * pow(p->loc-x, p->shape) );
  } else {
    double gamma_r = 1.0/p->sd_right*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
    return A * exp( -1.0 * pow(gamma_r, p->shape) * pow(x-p->loc, p->shape) );
  }
}


// **************************************************************************
// Class pdf_agg
// Evaluate the log-likelihood function
double pdf_agg::log_L( vector<double>& x ){
  size_t N = x.size();
  double gamma_a = 1.0/(p->sd_left+p->sd_right)*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double gamma_l = 1.0/p->sd_left*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double gamma_r = 1.0/p->sd_right*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double A = p->shape*gamma_a/gsl_sf_gamma(1.0/p->shape);

  double log_L = (double)N*log(A);
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( p->loc-x[i] > NEARLY_ZERO_AGG ){
      log_L -= pow((p->loc-x[i])*gamma_l, p->shape);
    } else if( x[i]-p->loc > NEARLY_ZERO_AGG ) {
      log_L -= pow((x[i]-p->loc)*gamma_r, p->shape);
    }
  }
  return log_L;
}


// **************************************************************************
// Class pdf_agg
// Evaluate the log-likelihood function
double pdf_agg::log_L( vector<double>& x, vector<double>& w ){
  if( x.size() != w.size() ){
    cerr << "Error! the number of the data points does not equal to "
	 << "the number of weights." << endl;
    exit(-1);
  }
  double cnt = 0.0;
  for( unsigned i=0 ; i<w.size() ; ++i ){
    cnt += w[i];
  }
  double gamma_a = 1.0/(p->sd_left+p->sd_right)*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double gamma_l = 1.0/p->sd_left*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double gamma_r = 1.0/p->sd_right*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double A = p->shape*gamma_a/gsl_sf_gamma(1.0/p->shape);

  double log_L = cnt * log(A);
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( p->loc-x[i] > NEARLY_ZERO_AGG ){
      log_L -= w[i] * pow((p->loc-x[i])*gamma_l, p->shape);
    } else if( x[i]-p->loc > NEARLY_ZERO_AGG ) {
      log_L -= w[i] * pow((x[i]-p->loc)*gamma_r, p->shape);
    }
  }
  return log_L;
}


// **************************************************************************
// Class pdf_agg
// Evaluate the derivative of the log-likelihood function
double pdf_agg::log_dL( vector<double>& x ){
  size_t N = x.size();
  double c = p->shape;
  double D2 = 0.5*(2.0/c-3.0*gsl_sf_psi(3.0/c)/pow(c,2)+3*gsl_sf_psi(1.0/c)/pow(c,2));
  double D3 = (gsl_sf_psi(1.0/c) - 3.0*gsl_sf_psi(3.0/c)) / (2*c);
  double log_dL = N*D2;
  double gamma_l = 1.0/p->sd_left*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double gamma_r = 1.0/p->sd_right*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( p->loc-x[i] > NEARLY_ZERO_AGG ){
      log_dL -= pow((p->loc-x[i])*gamma_l, c)*(log((p->loc-x[i])*gamma_l)+D3);
    } else if( x[i]-p->loc > NEARLY_ZERO_AGG ) {
      log_dL -= pow((x[i]-p->loc)*gamma_r, c)*(log((x[i]-p->loc)*gamma_r)+D3);
    }
  }
  return log_dL;
}


// **************************************************************************
// Class pdf_agg
// Evaluate the derivative of the log-likelihood function
double pdf_agg::log_dL( vector<double>& x, vector<double>& w ){
  if( x.size() != w.size() ){
    cerr << "Error! the number of the data points does not equal to "
	 << "the number of weights." << endl;
    exit(-1);
  }
  double cnt = 0.0;
  for( unsigned i=0 ; i<w.size() ; ++i ){
    cnt += w[i];
  }

  double c = p->shape;
  double D2 = 0.5*(2.0/c-3.0*gsl_sf_psi(3.0/c)/pow(c,2)+3*gsl_sf_psi(1.0/c)/pow(c,2));
  double D3 = (gsl_sf_psi(1.0/c) - 3.0*gsl_sf_psi(3.0/c)) / (2*c);
  double log_dL = cnt * D2;
  double gamma_l = 1.0/p->sd_left*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double gamma_r = 1.0/p->sd_right*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( p->loc-x[i] > NEARLY_ZERO_AGG ){
      log_dL -= w[i] * pow((p->loc-x[i])*gamma_l, c)*(log((p->loc-x[i])*gamma_l)+D3);
    } else if( x[i]-p->loc > NEARLY_ZERO_AGG ) {
      log_dL -= w[i] * pow((x[i]-p->loc)*gamma_r, c)*(log((x[i]-p->loc)*gamma_r)+D3);
    }
  }
  return log_dL;
}


// **************************************************************************
// Class pdf_agg
// Print sample points 
void pdf_agg::print_sample_points( string file, double min, double max ){
  ofstream ofs( file.c_str() );
  double width = (max-min)/(PLOT_SAMPLES_AGG-1);
  bool flg_right = false;
  for( double x=min ; x<max ; x+=width ){
    if( !flg_right && x>p->loc ){
      flg_right = true;
      ofs << p->loc << "\t" << eval(p->loc) << endl;
    }
    ofs << x << "\t" << eval(x) << endl;
  }
  ofs << max << "\t" << eval(max) << endl;
  ofs.close();
}


// **************************************************************************
// Class pdf_agg
// Return the sample median
double pdf_agg::cal_median( vector<double>& x ){
  if( x.size() % 2 == 0 ){
    size_t i = x.size()/2;
    size_t j = i+1;
    return (x[i] + x[j])/2.0;
  } else {
    size_t i = (x.size() + 1)/2;
    return x[i];
  }
}


// **************************************************************************
// Class pdf_agg
// Return the sample median
double pdf_agg::cal_median( vector<double>& x, vector<double>& w ){
  if( x.size() != w.size() ){
    cerr << "Error! the number of the data points does not equal to "
	 << "the number of weights." << endl;
    exit(-1);
  }
  double cnt = 0.0;
  for( unsigned i=0 ; i<w.size() ; ++i ){
    cnt += w[i];
  }
  double v_cnt = 0.0;
  for( unsigned i=0 ; i<w.size() ; ++i ){
    v_cnt += w[i];
    if( v_cnt >= cnt/2.0 ){
      return x[i];
    }
  }
  cerr << "Error! : Median cannot be found." << endl;
  exit(-1);
  return -1.0;
}


// **************************************************************************
// Class pdf_agg
// Return the mode estimated by the histogram method
double pdf_agg::cal_mode_hist( vector<double>& x ){
  histogram h( HIST_BIN_AGG, x );
  return h.get_mode();
}


// **************************************************************************
// Class pdf_agg
// Return the mode estimated by the histogram method
double pdf_agg::cal_mode_hist( vector<double>& x, vector<double>& w ){
  histogram h( HIST_BIN_AGG, x, w );
  return h.get_mode();
}


// **************************************************************************
// Class pdf_agg
// Return the invariant standard deviation of the left side
double pdf_agg::cal_sd_left( vector<double>& x ){
  double var = 0.0;
  size_t cnt = 0;
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( x[i] < p->loc ){
      var += pow(x[i]-p->loc,2);
      ++cnt;
    }
  }
  if(cnt <= 1){ cnt = 2; }
  return sqrt(var/(cnt-1));
}


// **************************************************************************
// Class pdf_agg
// Return the invariant standard deviation of the left side
double pdf_agg::cal_sd_left( vector<double>& x, vector<double>& w ){
  if( x.size() != w.size() ){
    cerr << "Error! the number of the data points does not equal to "
	 << "the number of weights." << endl;
    exit(-1);
  }
  double cnt = 0.0;
  double var = 0.0;
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( x[i] < p->loc ){
      cnt += w[i];
      var += w[i] * pow(x[i]-p->loc,2);
    }
  }
  if(cnt <= 1){ cnt = 2; }
  return sqrt(var/(cnt-1.0));
}


// **************************************************************************
// Class pdf_agg
// Return the invariant standard deviation of the right side
double pdf_agg::cal_sd_right( vector<double>& x ){
  double var = 0.0;
  size_t cnt = 0;
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( x[i] >= p->loc ){
      var += pow(x[i]-p->loc,2);
      ++cnt;
    }
  }
  if(cnt <= 1){ cnt = 2; }
  return sqrt(var/(cnt-1));
}


// **************************************************************************
// Class pdf_agg
// Return the invariant standard deviation of the right side
double pdf_agg::cal_sd_right( vector<double>& x, vector<double>& w ){
  if( x.size() != w.size() ){
    cerr << "Error! the number of the data points does not equal to "
	 << "the number of weights." << endl;
    exit(-1);
  }
  double cnt = 0.0;
  double var = 0.0;
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( x[i] >= p->loc ){
      cnt += w[i];
      var += w[i] * pow(x[i]-p->loc,2);
    }
  }
  if(cnt <= 1){ cnt = 2; }
  return sqrt(var/(cnt-1.0));
}


// **************************************************************************
// Class pdf_agg
// Return the local optimum for the shape parameter
// Computation is based on the FR method
double pdf_agg::cal_shape( vector<double>& x ){
  p->shape = MIN_SHAPE_AGG;
  if( log_dL( x ) < 0 )
    return MIN_SHAPE_AGG;

  const gsl_multimin_fdfminimizer_type* T 
    = gsl_multimin_fdfminimizer_conjugate_fr;
  // = gsl_multimin_fdfminimizer_vector_bfgs;
  gsl_multimin_fdfminimizer* s
    = gsl_multimin_fdfminimizer_alloc( T, 1 );

  gsl_multimin_function_fdf obj_func;
  gsl_param_agg gp;
  gp.first        = &x;
  gp.second       = p;
  obj_func.f      = &agg_f;
  obj_func.df     = &agg_df;
  obj_func.fdf    = &agg_fdf;
  obj_func.n      = 1;
  obj_func.params = &gp;
  gsl_vector* v   = gsl_vector_alloc(1); 
  gsl_vector_set(v, 0, p->shape);

  size_t iter = 0;
  gsl_multimin_fdfminimizer_set( s, &obj_func, v, STEP_SIZE_AGG, TOL_AGG );
  int status;
  do {
    ++iter;
    status = gsl_multimin_fdfminimizer_iterate(s);
    if( status ) break;
    status = gsl_multimin_test_gradient(s->gradient, EPSABS_AGG);
    double shape = gsl_vector_get(s->x,0);
    cerr << iter << "-th iteration (shape = " << shape << ")" << endl;
  } while ( status == GSL_CONTINUE && iter < ITER_AGG );
  double shape = gsl_vector_get(s->x, 0);
  cerr << iter << " steps." << endl;
  gsl_multimin_fdfminimizer_free(s);
  gsl_vector_free(v);

  return shape;
}


// **************************************************************************
// Class pdf_agg
// Return the local optimum for the shape parameter
// Computation is based on the FR method
double pdf_agg::cal_shape( vector<double>& x, vector<double>& w ){
  p->shape = MIN_SHAPE_AGG;
  if( log_dL( x, w ) < 0 )
    return MIN_SHAPE_AGG;

  const gsl_multimin_fdfminimizer_type* T 
    = gsl_multimin_fdfminimizer_conjugate_fr;
  // = gsl_multimin_fdfminimizer_vector_bfgs;
  gsl_multimin_fdfminimizer* s
    = gsl_multimin_fdfminimizer_alloc( T, 1 );

  gsl_multimin_function_fdf obj_func;
  gsl_param_w_agg w_gp;
  w_gp.push_back(&x);
  w_gp.push_back(&w);
  w_gp.push_back(p);
  obj_func.f      = &w_agg_f;
  obj_func.df     = &w_agg_df;
  obj_func.fdf    = &w_agg_fdf;
  obj_func.n      = 1;
  obj_func.params = &w_gp;
  gsl_vector* v = gsl_vector_alloc(1); 
  gsl_vector_set(v, 0, p->shape);

  size_t iter = 0;
  gsl_multimin_fdfminimizer_set( s, &obj_func, v, STEP_SIZE_AGG, TOL_AGG );
  int status;
  do {
    ++iter;
    status = gsl_multimin_fdfminimizer_iterate(s);
    if( status ) break;
    status = gsl_multimin_test_gradient(s->gradient, EPSABS_AGG);
    double shape = gsl_vector_get(s->x,0);
    cerr << iter << "-th iteration (shape = " << shape << ")" << endl;
  } while ( status == GSL_CONTINUE && iter < ITER_AGG );
  double shape = gsl_vector_get(s->x, 0);
  cerr << iter << " steps." << endl;
  gsl_multimin_fdfminimizer_free(s);
  gsl_vector_free(v);

  return shape;
}


// **************************************************************************
// Function needed to perform the optimization of the shape parameter (f)
double agg_f( const gsl_vector* v, void* gsl_param ){
  gsl_param_agg    gp   = *(gsl_param_agg*)gsl_param;
  vector<double>* data  = gp.first;
  param_agg*      p_agg = gp.second;
  double shape = gsl_vector_get(v, 0);
  p_agg->set_shape( shape );
  pdf_agg agg;
  agg.set_param( p_agg );

  return -1.0 * agg.log_L( *data );
};


// **************************************************************************
// Function needed to perform the optimization of the shape parameter (df)
void agg_df( const gsl_vector* v, void* gsl_param, gsl_vector* df ){
  gsl_param_agg    gp   = *(gsl_param_agg*)gsl_param;
  vector<double>* data  = gp.first;
  param_agg*      p_agg = gp.second;
  double shape = gsl_vector_get(v, 0);
  p_agg->set_shape( shape );
  pdf_agg agg;
  agg.set_param( p_agg );
  double log_dL = agg.log_dL( *data );
  gsl_vector_set( df, 0, -1.0*log_dL );
}


// **************************************************************************
// Function needed to perform the optimization of the shape parameter (fdf)
void agg_fdf( const gsl_vector* v, void* data, double* f, gsl_vector* df ){
  *f = agg_f( v, data );
  agg_df( v, data, df );
}


// **************************************************************************
// Function needed to perform the optimization of the shape parameter (f)
double w_agg_f( const gsl_vector* v, void* gsl_param ){
  gsl_param_w_agg w_gp   = *(gsl_param_w_agg*)gsl_param;
  vector<double>* data   = (vector<double>*)w_gp[0];
  vector<double>* weight = (vector<double>*)w_gp[1];
  param_agg*      p_agg  = (param_agg*)w_gp[2];
  double shape = gsl_vector_get(v, 0);
  p_agg->set_shape( shape );
  pdf_agg agg;
  agg.set_param( p_agg );
  return -1.0 * agg.log_L( *data, *weight );
};


// **************************************************************************
// Function needed to perform the optimization of the shape parameter (df)
void w_agg_df( const gsl_vector* v, void* gsl_param, gsl_vector* df ){
  gsl_param_w_agg w_gp   = *(gsl_param_w_agg*)gsl_param;
  vector<double>* data   = (vector<double>*)w_gp[0];
  vector<double>* weight = (vector<double>*)w_gp[1];
  param_agg*      p_agg  = (param_agg*)w_gp[2];
  double shape = gsl_vector_get(v, 0);
  p_agg->set_shape( shape );
  pdf_agg agg;
  agg.set_param( p_agg );
  double log_dL = agg.log_dL( *data, *weight );
  gsl_vector_set( df, 0, -1.0*log_dL );
}


// **************************************************************************
// Function needed to perform the optimization of the shape parameter (fdf)
void w_agg_fdf( const gsl_vector* v, void* data, double* f, gsl_vector* df ){
  *f = w_agg_f( v, data );
  w_agg_df( v, data, df );
}
