// *************************************************************************
//  dpd_cluster.cpp
// *************************************************************************

#include "dpd_cluster.h"
using namespace std;


// *************************************************************************
// Class dpd_cluster
// Constructor
dpd_cluster::dpd_cluster( ortholog_set& os, position_2d_map& pos_2d_map, 
			  double col_index, unsigned t_dpd ){
  map<string, double>::iterator os_it = os.begin();
  unsigned next_cid = 1;
  progress_bar bar( os.size() );
  unsigned cnt = 0;
  while( os_it != os.end() ){
    bar.set_prog( ++cnt );
    string gp = os_it->first;
    unsigned min_cid = get_min_cid( gp, pos_2d_map, col_index, 
				    t_dpd, next_cid );
    append( gp, min_cid );
    merge_cluster( gp, pos_2d_map, col_index, t_dpd, min_cid );
    ++os_it;
  }
}


// *************************************************************************
// Class dpd_cluster
// Return the minimum cluster ID that contains neighbors
unsigned dpd_cluster::get_min_cid( string gp, position_2d_map& pos_2d_map, 
				   double col_index, 
				   unsigned t_dpd, unsigned& next_cid ){
  map< unsigned, list<string> >::iterator it = cid2gp_list.begin();
  while( it != cid2gp_list.end() ){
    unsigned cid = it->first;
    if( is_neighbor( gp, it->second, pos_2d_map, col_index, t_dpd ) )
      return cid;
    ++it;
  }
  return next_cid++;
}


// *************************************************************************
// Class dpd_cluster
// Return whether neighbors exist or not in a given list
bool dpd_cluster::is_neighbor( string gp, list<string>& gp_list, 
			       position_2d_map& pos_2d_map, 
			       double col_index, unsigned t_dpd ){
  list<string>::iterator it = gp_list.begin();
  while( it != gp_list.end() ){
    string cur_gp = *it;
    double dpd = pos_2d_map.DPD( gp, cur_gp, col_index );
    if( dpd != INF and dpd <= t_dpd ){
      return true;
    }
    ++it;
  }
  return false;
}


// *************************************************************************
// Class dpd_cluster
// Append a gene pair
void dpd_cluster::append( string gp, unsigned min_cid ){
  map< unsigned, list<string> >::iterator it = cid2gp_list.find( min_cid );
  if( it == cid2gp_list.end() ){
    list<string> gp_list;
    gp_list.push_back( gp );
    cid2gp_list.insert( map< unsigned, list<string> >
			::value_type( min_cid, gp_list ) );
  } else {
    it->second.push_back( gp );
  }
  gp2cid.erase( gp );
  gp2cid.insert( map<string, unsigned>::value_type( gp, min_cid ) );
}


// *************************************************************************
// Class dpd_cluster
// Merge clusters
void dpd_cluster::merge_cluster( string gp, position_2d_map& pos_2d_map, 
				 double col_index, 
				 unsigned t_dpd, unsigned min_cid ){
  map< unsigned, list<string> >::iterator it = cid2gp_list.find( min_cid );
  if( it == cid2gp_list.end() ){
    cerr << "Error!" << endl;
    exit(-1);
  }
  list<string>* base_list = &(it->second);

  it = cid2gp_list.begin();
  while( it != cid2gp_list.end() ){
    unsigned cur_cid = it->first;
    if( cur_cid <= min_cid ){
      ++it;
      continue;
    }
    if( is_neighbor( gp, it->second, pos_2d_map, col_index, t_dpd ) ){
      base_list->merge( it->second );
      change_cid( it->second, min_cid );
      ++it;
      cid2gp_list.erase( cur_cid );
    } else {
      ++it;
    }
  }
}


// *************************************************************************
// Class dpd_cluster
// Change the cluster ID 
void dpd_cluster::change_cid( list<string>& gp_list, unsigned cid ){
  list<string>::iterator list_it = gp_list.begin();
  while( list_it != gp_list.end() ){
    string gp = *list_it;
    gp2cid.erase( gp );
    gp2cid.insert( map<string, unsigned>::value_type( gp, cid ) );
    ++list_it;
  }
}


// *************************************************************************
// Class dpd_cluster
// Return the size 
size_t dpd_cluster::size(){
  size_t size = 0;
  map< unsigned, list<string> >::iterator it = cid2gp_list.begin();
  while( it != cid2gp_list.end() ){
    size += it->second.size();
    ++it;
  }
  return size;
}


// *************************************************************************
// Class dpd_cluster
// Return the number of the clusters
size_t dpd_cluster::num_cluster(){
  return cid2gp_list.size();
}


// *************************************************************************
// Class dpd_cluster
// Return the number of the isolated clusters
size_t dpd_cluster::num_isolated(){
  size_t num_iso = 0;
  map< unsigned, list<string> >::iterator it = cid2gp_list.begin();
  while( it != cid2gp_list.end() ){
    size_t size = it->second.size();
    if( size == 1 ) ++num_iso;
    ++it;
  }
  return num_iso;
}


// *************************************************************************
// Class dpd_cluster
// Print the results of clustering
void dpd_cluster::print( gene_set& gs1, gene_set& gs2, string outfile ){
  ofstream ofs( outfile.c_str() );
  unsigned cluster_id = 1;
  map< unsigned, list<string> >::iterator it = cid2gp_list.begin();
  while( it != cid2gp_list.end() ){
    ofs << "Cluster ID: " << cluster_id++ << endl;
    list<string>* p_list = &(it->second);
    list<string>::iterator list_it = p_list->begin();
    while( list_it != p_list->end() ){
      string id_pair = *list_it;
      list<string> id_list = split( id_pair, " " );
      list<string>::iterator id_it = id_list.begin();
      string id_1  = *id_it; ++id_it;
      string id_2  = *id_it; ++id_it;
      string str_1 = gs1.get_str( id_1, ":" );
      string str_2 = gs2.get_str( id_2, ":" );
      ofs << str_1 << "\t" << str_2 << endl;
      ++list_it;
    }
    ofs << endl;
    ++it;
  }
  ofs.close();
}


// *************************************************************************
// Class dpd_cluster
// Return the average of the cluster size
double dpd_cluster::average_cluster_size(){
  return (double)size()/num_cluster();
}


// *************************************************************************
// Class dpd_cluster
// Return the standard deviation of the cluster size
double dpd_cluster::sd_cluster_size(){
  double ave_size = average_cluster_size();
  double sq_sum   = 0.0;
  map< unsigned, list<string> >::iterator it = cid2gp_list.begin();
  while( it != cid2gp_list.end() ){
    size_t c_size = it->second.size();
    sq_sum += pow( c_size - ave_size, 2 );
    ++it;
  }
  return sqrt( sq_sum/num_cluster() );
}


// *************************************************************************
// Class dpd_cluster
// Return the frequency of the cluster size
map<size_t, size_t> dpd_cluster::size_frequency(){
  map<size_t, size_t> size_freq;
  map< unsigned, list<string> >::iterator it = cid2gp_list.begin();
  while( it != cid2gp_list.end() ){
    size_t c_size = it->second.size();
    map<size_t, size_t>::iterator sf_it = size_freq.find( c_size );
    if( sf_it == size_freq.end() ){
      size_freq.insert( map<size_t, size_t>::value_type( c_size, 1 ) );
    } else {
      size_t cur_freq = sf_it->second;
      size_freq.erase( c_size );
      size_freq.insert( map<size_t, size_t>::value_type(c_size, ++cur_freq) );
    }
    ++it;
  }
  return size_freq;
}


// *************************************************************************
// Class dpd_cluster
// Print the isolated ortholog pairs
void dpd_cluster::print_iso( c_hit_set& c_hs, gene_set& gs1, gene_set& gs2, 
			     string outfile ){
  ofstream ofs( outfile.c_str() );
  map< unsigned, list<string> >::iterator it = cid2gp_list.begin();
  while( it != cid2gp_list.end()  ){
    list<string>* p_list = &(it->second);
    size_t c_size = p_list->size();
    if( c_size == 1 ){
      string id_pair = *(p_list->begin());
      double score   = c_hs.get_bit( id_pair );
      list<string> id_list = split( id_pair, " " );
      list<string>::iterator id_it = id_list.begin();
      string id_1  = *id_it; ++id_it;
      string id_2  = *id_it; ++id_it;
      string str_1 = gs1.get_str( id_1, ":" );
      string str_2 = gs2.get_str( id_2, ":" );
      ofs << str_1 << "\t" << str_2 << "\t" << score << endl;
    }
    ++it;
  }
  ofs.close();
}


// *************************************************************************
// Class dpd_cluster
// Print the non-isolated ortholog pairs
void dpd_cluster::print_non_iso( c_hit_set& c_hs, gene_set& gs1, 
				 gene_set& gs2, string outfile ){
  ofstream ofs( outfile.c_str() );
  map< unsigned, list<string> >::iterator it = cid2gp_list.begin();
  while( it != cid2gp_list.end()  ){
    list<string>* p_list = &(it->second);
    size_t c_size = p_list->size();
    if( c_size == 1 ){ ++it; continue; }
    list<string>::iterator p_it = p_list->begin();
    while( p_it != p_list->end() ){
      string id_pair = *p_it;
      double score   = c_hs.get_bit( id_pair );
      list<string> id_list = split( id_pair, " " );
      list<string>::iterator id_it = id_list.begin();
      string id_1  = *id_it; ++id_it;
      string id_2  = *id_it; ++id_it;
      string str_1 = gs1.get_str( id_1, ":" );
      string str_2 = gs2.get_str( id_2, ":" );
      ofs << str_1 << "\t" << str_2 << "\t" << score << endl;
      ++p_it;
    }
    ++it;
  }
  ofs.close();
}


// *************************************************************************
// Class dpd_cluster
// Print the details of the results
void dpd_cluster::print_details( c_hit_set& c_hs, 
				 gene_set& gs1, gene_set& gs2, 
				 string outfile ){
  ofstream ofs( outfile.c_str() );
  map< unsigned, list<string> >::iterator it = cid2gp_list.begin();
  while( it != cid2gp_list.end()  ){
    list<string>* p_list = &(it->second);
    list<string>::iterator list_it = p_list->begin();
    while( list_it != p_list->end() ){
      string id_pair = *list_it;
      double score   = c_hs.get_bit( id_pair );
      list<string> id_list = split( id_pair, " " );
      list<string>::iterator id_it = id_list.begin();
      string id_1  = *id_it; ++id_it;
      string id_2  = *id_it; ++id_it;
      string str_1 = gs1.get_str( id_1, ":" );
      string str_2 = gs2.get_str( id_2, ":" );
      ofs << str_1 << "\t" << str_2 << "\t" << score << endl;
      ++list_it;
    }
    ++it;
  }
  ofs.close();
}


// *************************************************************************
// Class dpd_cluster
// Print the status of the results of clustering
void dpd_cluster::print_stats( string outfile ){
  ofstream ofs( outfile.c_str() );
  double ave_c_size = average_cluster_size();
  double sd_c_size  = sd_cluster_size();
  size_t num_iso    = num_isolated();
  ofs << "Average cluster size  = " << ave_c_size <<" +- "<< sd_c_size  << endl
      << "# of orthologs        = " << size()                           << endl
      << "# of clusters         = " << num_cluster()                    << endl
      << "# of isolated cluster = " << num_iso                          << endl
      << endl
      << "Cluster size frequency:" << endl;
  map<size_t, size_t> size_freq = size_frequency();
  map<size_t, size_t>::iterator sf_it = size_freq.begin();
  while( sf_it != size_freq.end() ){
    size_t c_size = sf_it->first;
    size_t freq   = sf_it->second;
    ofs << " " << c_size << "\t" << freq << endl;
    ++sf_it;
  }
  ofs.close();
}
