// *************************************************************************
//  gene_set.cpp
// *************************************************************************

#include "gene_set.h"
using namespace std;


// *************************************************************************
// Class gene_set
// Return the next ID 
unsigned gene_set::next_id(){
  return chr2id.size()+1;
}


// *************************************************************************
// Class gene_set
// Constructor
gene_set::gene_set( string in_file ){
  try {
    ifstream ifs( in_file.c_str() );
    if( ifs.fail() )
      throw (string)"File open error: " + in_file.c_str();
    string str;
    while( getline(ifs, str) ){
      list<string> cols = split( str, "\t" );
      if( cols.size() != 5 )
	throw "Inappropriate file format: " + str;
      gene g( str, "\t" );
      insert( g );
    }
    ifs.close();
  } catch ( string err_msg ){
    cerr << err_msg << endl; 
    exit(-1);
  }
}


// *************************************************************************
// Class gene_set
// Insert new gene
void gene_set::insert( gene g ){
  insert( g.get_ID(), g );
}


// *************************************************************************
// Class gene_set
// Insert new gene
void gene_set::insert( string id, gene g ){
  ID2gene.insert( map<string, gene>::value_type( id, g ) );
  map<string, unsigned>::iterator it = chr2id.find( g.get_chr() );
  if( it == chr2id.end() ){
    unsigned id = next_id();
    chr2id.insert( map<string, unsigned>::value_type( g.get_chr(), id ) );
  }
}


// *************************************************************************
// Class gene_set
// Return gene specified by gene ID
gene gene_set::get_gene( string s ){
  map<string, gene>::iterator it = ID2gene.find(s);
  if( it == ID2gene.end() ){
    cerr << "warning: No gene is found." << endl;
    exit(1);
  }
  return it->second;
}


// *************************************************************************
// Class gene_set
// Return gene string specified by gene ID
string gene_set::get_str( string s, string delim ){
  gene g = get_gene(s);
  return g.get_str(delim);
}


// *************************************************************************
// Class gene_set
// Return locus string specified by gene ID
string gene_set::get_locus( string s, string delim ){
  gene  g   = get_gene(s);
  locus loc = g.get_locus();
  return loc.get_str( chr2id, delim );
}


// *************************************************************************
// Class gene_set
// Print IDs in the gene set
void gene_set::print_IDs(){
  map<string, gene>::iterator it = ID2gene.begin();
  while( it != ID2gene.end() ){
    cout << it->first << endl;
    ++it;
  }
}


// *************************************************************************
// Class gene_set
// Print gene strings
void gene_set::print_strs(){
  map<string, gene>::iterator it = ID2gene.begin();
  while( it != ID2gene.end() ){
    cout << it->second.get_str() << endl;
    ++it;
  }
}


// *************************************************************************
// Class gene_set
// Print genes
void gene_set::print( string outfile, string delim ){
  ofstream ofs( outfile.c_str() );
  map<string, gene>::iterator it = ID2gene.begin();
  while( it != ID2gene.end() ){
    ofs << it->second.get_str(delim) << endl;
    ++it;
  }
  ofs.close();
}


// *************************************************************************
// Extract genes included in the hit set
gene_set genes_in_hit_set( c_hit_set& c_hs, gene_set& gs, bool flg ){
  gene_set c_gs;
  map<string, c_hit>::iterator hs_it = c_hs.begin();
  while( hs_it != c_hs.end() ){
    string ip = hs_it->first;
    list<string> IDs = split( ip, " " );
    list<string>::iterator id_it = IDs.begin();
    string id_1 = *id_it; ++id_it;
    string id_2 = *id_it; ++id_it;
    string id   = flg ? id_1 : id_2;
    map<string, gene>::iterator gs_it = gs.find( id );
    if( gs_it == gs.end() ){
      cerr << "Error: " << id << " is not found." << endl;
      exit(-1);
    }
    gene g = gs_it->second;
    c_gs.insert( id, g );
    ++hs_it;
  }
  return c_gs;
}
