// **************************************************************************
//  histogram.cpp
// **************************************************************************

#include "histogram.h"

// **************************************************************************
// Class histogram
// Return maximum value
double histogram::find_max( vector<double>& val_list ){
  double max_val = val_list[0];
  for( unsigned i=1 ; i<val_list.size() ; ++i ){
    if( val_list[i]>max_val ) max_val=val_list[i];
  }
  return max_val;
}


// **************************************************************************
// Class histogram
// Return minimum value
double histogram::find_min( vector<double>& val_list ){
  double min_val = val_list[0];
  for( unsigned i=1 ; i<val_list.size() ; ++i ){
    if( val_list[i]<min_val ) min_val=val_list[i];
  }
  return min_val;
}


// **************************************************************************
// Class histogram
// Constructor
histogram::histogram( size_t n, vector<double>& val_list ){
  hist_size = n;
  h = gsl_histogram_alloc( hist_size ); 
  min     = find_min( val_list );
  obs_max = find_max( val_list );
  max     = obs_max + (obs_max-min)/(hist_size*10);
  gsl_histogram_set_ranges_uniform( h, min, max );
  for( unsigned i=0 ; i<val_list.size() ; ++i ){
    gsl_histogram_increment(h, val_list[i]);
  }
}


// **************************************************************************
// Class histogram
// Constructor
histogram::histogram( size_t n, vector<double>& val_list, 
		      vector<double>& w_list ){
  hist_size = n;
  h = gsl_histogram_alloc( hist_size ); 
  min     = find_min( val_list );
  obs_max = find_max( val_list );
  max     = obs_max + (obs_max-min)/(hist_size*10);
  if(min == max){ max += SMALL_VALUE_HISTOGRAM; }
  gsl_histogram_set_ranges_uniform( h, min, max );
  for( unsigned i=0 ; i<val_list.size() ; ++i ){
    gsl_histogram_accumulate(h, val_list[i], w_list[i]);
  }
}


// **************************************************************************
// Class histogram
// Return the mode estimated by this histogram
double histogram::get_mode(){
  size_t max_bin = 1;
  double max_val = gsl_histogram_get( h, max_bin );
  for( size_t bin=2 ; bin<=(gsl_histogram_bins(h)-1) ; ++bin ){
    double cur_val = gsl_histogram_get( h, bin );
    if( cur_val > max_val ){
      max_bin = bin;
      max_val = cur_val;
    }
  }
  double lower = 0.0;
  double upper = 0.0;
  gsl_histogram_get_range( h, max_bin, &lower, &upper );
  if( max_bin == gsl_histogram_bins(h)-1 ){
    return upper;
  } else {
    return (upper+lower)/2.0;
  }
}


// **************************************************************************
// Class histogram
// Return vector of points
vector< point > histogram::get_points(){
  return get_points( 0, hist_size-1 );
}


// **************************************************************************
// Class histogram
// Return vector of points
vector< point > histogram::get_points( unsigned stt, unsigned end ){
  vector< point > points;
  for( unsigned i=stt ; i<=end ; ++i ){
    double freq, lower, upper;
    freq = gsl_histogram_get( h, i );
    gsl_histogram_get_range( h, i, &lower, &upper );
    point p = point::pair( (lower+upper)/2.0, freq );
    points.push_back( p );
  }
  return points;
}


// **************************************************************************
// Class histogram
// Print this histogram to file
void histogram::print( string outfile, bool flg_prob ){
  print( outfile, 0, hist_size-1, flg_prob );
}


// **************************************************************************
// Class histogram
// Print this histogram to file
void histogram::print( string outfile, unsigned stt, unsigned end, 
		       bool flg_prob ){
  ofstream ofs( outfile.c_str() );
  vector< point > points = get_points( stt, end );
  for( unsigned i=0 ; i<points.size() ; ++i ){
    if( flg_prob ){
      ofs << points[i].first << "\t" << points[i].second/points.size() << endl;
    } else {
      ofs << points[i].first << "\t" << points[i].second << endl;
    }
  }
  ofs.close();
}


// **************************************************************************
// Class histogram
// Draw histogram
void histogram::draw( string name, bool flg_prob ){
  draw( name, 0, hist_size-1, flg_prob );
}


// **************************************************************************
// Class histogram
// Draw histogram
void histogram::draw( string name, unsigned stt, unsigned end, bool flg_prob ){
  vector< point > points = get_points();
  string datfile = name + ".hist.dat";
  string gnufile = name + ".hist.gnu";
  string epsfile = name + ".hist.eps";

  print( datfile, stt, end, flg_prob );
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript"                            << endl
      << "set output \"" << epsfile << "\""                   << endl
      << "set style fill solid 0.7"                           << endl
      << "set xtics nomirror"                                 << endl
      << "set ytics nomirror"                                 << endl
    //<< "set style fill pattern 2"                           << endl
      << "plot \"" << datfile << "\" with boxes"              << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
}


// **************************************************************************
// Class histogram
// Draw two histograms
void histogram::draw( histogram& h1, string name, bool flg_prob ){
  draw( h1, name, 0, hist_size-1, flg_prob );
}


// **************************************************************************
// Class histogram
// Draw two histograms
void histogram::draw( histogram& h1, string name, unsigned stt, unsigned end, 
		      bool flg_prob ){
  vector< point > points = get_points();
  string datfile_0 = name + ".hist.0.dat";
  string datfile_1 = name + ".hist.1.dat";
  string gnufile   = name + ".hist.gnu";
  string epsfile   = name + ".hist.eps";

  print( datfile_0, stt, end, flg_prob );
  h1.print( datfile_1, stt, end, flg_prob );
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript color"                            << endl
      << "set output \"" << epsfile << "\""                         << endl
      << "set boxwidth 0.9 absolute"                                << endl
      << "set style fill solid 1.00"                                << endl
      << "set style histogram clustered gap 1"                      << endl
      << "set style data histograms"                                << endl
      << "plot \"" << datfile_0 << "\" using 2 axis x1y1, "
      << "\"" << datfile_1 << "\" using 2 axis x1y2"                << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
}
