// *************************************************************************
//  model.cpp
// *************************************************************************

#include "model.h"
using namespace std;


// *************************************************************************
// Class model
// Constructor
model::model( vector<double>& x, vector<double>& w ){
  cerr << "--- OGG model ---" << endl;
  ogg.set_param( &p_ogg );
  ogg.fit_ml( x, w );
  p_ogg.show();
  cerr << "-----------------" << endl;
  cerr << "--- AGG model ---" << endl;
  agg.set_param( &p_agg );
  agg.fit_ml( x, w );
  p_agg.show();
  cerr << "-----------------" << endl;
  cerr << "-----------------" << endl;
  if( ogg.aic(x, w) > agg.aic(x, w) ){
    cerr << "OGG model is selected" << endl;
    flg_model = false;
  } else {
    cerr << "AGG model is selected" << endl;
    flg_model = true;
  }
  cerr << "-----------------" << endl;
}


// *************************************************************************
// Class model
// Evaluate the probability density
double model::eval( double x ){
  if( flg_model )
    return agg.eval(x);
  else 
    return ogg.eval(x);
}


// *************************************************************************
// Class model
// Draw the PDF with histogram
void model::draw_pdf( string name, vector<double>& x, vector<double>& w ){
  if( flg_model )
    agg.plot( name, x, w, HIST_BIN_MODEL );
  else 
    ogg.plot( name, x, w, HIST_BIN_MODEL );
}


// *************************************************************************
// Return the score
double calc_score( model& m1, model& m0, double x ){
  return log( m1.eval(x) / m0.eval(x) );
}


// *************************************************************************
// Return the monotoniaclly increasing score
double calc_mi_score( model& m1, model& m0, double x, double peak_val ){
  if( x <= peak_val )
    return calc_score( m1, m0, x );
  else 
    return calc_score( m1, m0, peak_val ) + SLOPE_SCORE * (x-peak_val);
}


// *************************************************************************
// Return the certainity between two models
double calc_certainity( vector<double>& x, model& m1, model& m0 ){
  double sum = 0.0;
  size_t num = 0;
  for(unsigned i=0 ; i<x.size() ; ++i ){
    double p1   = m1.eval(x[i]);
    double p0   = m0.eval(x[i]);
    double p_c1 = p1 / (p1+p0);
    double p_c0 = 1.0 - p_c1;
    double entropy = -1 * ( p_c1 * log2( p_c1 ) + p_c0 * log2( p_c0 ) );
    sum += 1 - entropy;
    ++num;
  }
  double certainity = sum / num;
  return certainity;
}


// *************************************************************************
// Return the J-divergence between two models
double calc_J_divergence( model& m1, model& m2, double min, double max ){
  double J_divergence = 0.0;
  double delta = (max-min)/INTEGRAL_PARTITION;
  double x;
  for( x=min ; x<max ; x+=delta ){
    double p1  = m1.eval(x);
    double p1_ = m1.eval(x+delta);
    double p2  = m2.eval(x);
    double p2_ = m2.eval(x+delta);
    double i_val  = (p1-p2)*log2(p1/p2);
    double i_val_ = (p1_-p2_)*log2(p1_/p2_);
    J_divergence += delta * 0.5 * ( i_val_ + i_val );
  }
  double p1  = m1.eval(x);
  double p1_ = m1.eval(max);
  double p2  = m2.eval(x);
  double p2_ = m2.eval(max);
  double i_val  = (p1-p2)*log2(p1/p2);
  double i_val_ = (p1_-p2_)*log2(p1_/p2_);
  J_divergence += delta * 0.5 * ( i_val_ + i_val );
  
  return J_divergence;
}


// *************************************************************************
// Draw histograms via GNUplot
void draw_histograms( string name, vector<double>& x, 
		      vector<double>& w0, vector<double>& w1 ){
  draw_histogram( name+".seeds", x, w1 );
  draw_histogram( name+".non_seeds", x, w0 );
}


// *************************************************************************
// Draw histogram via GNUplot
void draw_histogram( string name, vector<double>& x, vector<double>& w ){
  histogram h( HIST_BIN_MODEL, x, w );
  h.draw( name );
}


// *************************************************************************
// Draw score function via GNUplot
void draw_score_function( string name, model& m1, model& m0, 
			  double min, double max, size_t num_bin ){
  string pd_file = name + ".score_function.dat";
  string gnufile = name + ".score_function.gnu";
  string epsfile = name + ".score_function.eps";
  print_sample_points( pd_file, m1, m0, min, max, num_bin );
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript"                                      << endl
      << "set output \"" << epsfile << "\""                             << endl
      << "set xrange [" << min << ":" << max << "]"                     << endl
      << "set xtics nomirror"                                           << endl
      << "set ytics nomirror"                                           << endl
      << "plot \"" << pd_file << "\" with lines "
      << "title \"score function\" linewidth 2"
      << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
}


// *************************************************************************
// Draw monotonically increasing score function via GNUplot
void draw_mi_score_function( string name, model& m1, model& m0, 
			     double min, double max, double peak_val, 
			     size_t num_bin ){
  string pd_file = name + ".score_function.dat";
  string gnufile = name + ".score_function.gnu";
  string epsfile = name + ".score_function.eps";
  print_sample_points_mi( pd_file, m1, m0, min, max, peak_val, num_bin );
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript"                                      << endl
      << "set output \"" << epsfile << "\""                             << endl
      << "set xrange [" << min << ":" << max << "]"                     << endl
      << "set xtics nomirror"                                           << endl
      << "set ytics nomirror"                                           << endl
      << "plot \"" << pd_file << "\" with lines "
      << "title \"score function\" linewidth 2"
      << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
}


// *************************************************************************
// Draw two score functions via GNUplot
void draw_two_score_function( string name, model& m1, model& m0, 
			      double min, double max, double peak_val, 
			      size_t num_bin ){
  string pd_file_nor = name + ".normal_score_function.dat";
  string pd_file_mi  = name + ".mi_score_function.dat";
  string gnufile     = name + ".score_function.gnu";
  string epsfile     = name + ".score_function.eps";
  print_sample_points( pd_file_nor, m1, m0, min, max, num_bin );
  print_sample_points_mi( pd_file_mi, m1, m0, min, max, peak_val, num_bin );
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript color"                                << endl
      << "set output \"" << epsfile << "\""                             << endl
      << "set xrange [" << min << ":" << max << "]"                     << endl
      << "set xtics nomirror"                                           << endl
      << "set ytics nomirror"                                           << endl
      << "plot \"" << pd_file_nor << "\" with lines "
      << "title \"normal score function\" linewidth 2"
      << ", \"" << pd_file_mi << "\" with lines "
      << "title \"monotonically increasing score function\" linewidth 2"
      << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
}


// *************************************************************************
// Print sample points 
void print_sample_points( string file, model& m1, model& m0, 
			  double min, double max, size_t num_bin ){
  ofstream ofs( file.c_str() );
  double width = (max-min)/(num_bin-1);
  for( double x=min ; x<max ; x+=width ){
    ofs << x << "\t" << calc_score( m1, m0, x ) << endl;
  }
  ofs << max << "\t" << calc_score( m1, m0, max ) << endl;
  ofs.close();
}


// *************************************************************************
// Print sample points
void print_sample_points_mi( string file, model& m1, model& m0, 
			     double min, double max, double peak_val, 
			     size_t num_bin ){
  ofstream ofs( file.c_str() );
  double width = (max-min)/(num_bin-1);
  for( double x=min ; x<max ; x+=width ){
    ofs << x << "\t" << calc_mi_score( m1, m0, x, peak_val ) << endl;
  }
  ofs << max << "\t" << calc_mi_score( m1, m0, max, peak_val ) << endl;
  ofs.close();
}


