// *************************************************************************
//  oasys_hit_set.cpp
// *************************************************************************

#include "oasys_hit_set.h"
using namespace std;

// *************************************************************************
// Class oasys_hit_set
// Set this class
void oasys_hit_set::set( c_hit_set& c_hs, position_2d_map& pos_2d_map, 
			 ortholog_set& seeds, double col_index, 
			 double sds, double t_dpd ){
  map<string, c_hit>::iterator hs_it = c_hs.begin();
  progress_bar bar( c_hs.size() );
  unsigned cnt = 0;
  while( hs_it != c_hs.end() ){
    bar.set_prog( ++cnt );
    string id_pair = hs_it->first;
    c_hit  ch      = hs_it->second;
    double wnnso   = calc_wnnso( id_pair, pos_2d_map, seeds, 
				 col_index, sds, t_dpd );
    oasys_hit o_hit( ch, wnnso );
    insert( id_pair, o_hit );
    ++hs_it;
  }
  set_stats();
}


// *************************************************************************
// Class oasys_hit_set
// Set this class
void oasys_hit_set::set( c_hit_set& c_hs, position_2d_map& pos_2d_map, 
			 ortholog_set& seeds, double col_index, 
			 double sds, double t_dpd, bool flg ){
  flg = true;
  map<string, c_hit>::iterator hs_it = c_hs.begin();
  while( hs_it != c_hs.end() ){
    string id_pair = hs_it->first;
    c_hit  ch      = hs_it->second;
    double wnnso   = calc_wnnso( id_pair, pos_2d_map, seeds, 
				 col_index, sds, t_dpd );
    oasys_hit o_hit( ch, wnnso );
    insert( id_pair, o_hit );
    ++hs_it;
  }
  set_stats();
}


// *************************************************************************
// Class oasys_hit_set
// Set this class
void oasys_hit_set::set( string f ){
  try {
    ifstream ifs(f.c_str());
    if( ifs.fail() )
      throw (string)"File open error: " + f.c_str();
    string str;
    while( getline(ifs, str) ){
      oasys_hit o_hit( str );
      string id_pair = o_hit.get_IDpair();
      insert( id_pair, o_hit );
    }
  } catch ( string err_msg ){
    cerr << err_msg << endl; 
    exit(-1);
  }
  set_stats();
}


// *************************************************************************
// Class oasys_hit_set
// Calculate WNNSO value
void oasys_hit_set::set_stats(){
  double min = 0.0;
  double max = 0.0;
  double ave = 0.0;
  double sum = 0.0;
  map<string, oasys_hit>::iterator it = begin();
  bool   flg = true;
  while( it != end() ){
    double wnnso = it->second.get_wnnso();
    if( flg ) { max = wnnso; min = wnnso; flg = false; }
    sum += wnnso;
    if( wnnso > max ) { max = wnnso; }
    if( wnnso < min ) { min = wnnso; }
    ++it;
  }
  ave = sum / size();
  max_wnnso = max;
  min_wnnso = min;
  ave_wnnso = ave;
}


// *************************************************************************
// Class oasys_hit_set
// Calculate WNNSO value
double oasys_hit_set::calc_wnnso( string id_pair, 
				  position_2d_map& pos_2d_map, 
				  ortholog_set& seeds, 
				  double col_index, double sds, double t_dpd ){
  double wnnso = 0.0;
  string chr_pair = pos_2d_map.get_chrom_pair( id_pair );
  position_matrix* p_mat = pos_2d_map.get_position_matrix( chr_pair );
  map< string, mat_idx >::iterator mat_it = p_mat->id_begin();
  while( mat_it != p_mat->id_end() ){
    string cur_id_pair = mat_it->first;
    if( seeds.exist( cur_id_pair ) ){
      unsigned dpd = pos_2d_map.DPD( id_pair, cur_id_pair, col_index );
      if( dpd != INF and dpd<=t_dpd ){
	double weight = dpd2weight( dpd, sds );
	wnnso += weight;
      }
    }
    ++mat_it;
  }
  return wnnso;
}


// *************************************************************************
// Class oasys_hit_set
// Return weight value
double oasys_hit_set::dpd2weight( unsigned dpd, double sds ){
  double weight = 1 / ( sds * sqrt( 2 * M_PI ) ) 
    * exp( -1 * (double)dpd / ( 2 * sds * sds ) );
  return weight;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the WNNSO values
vector<double> oasys_hit_set::get_wnnso_list(){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    val_list.push_back( it->second.get_wnnso() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the WNNSO values
vector<double> oasys_hit_set::get_wnnso_list( ortholog_set& os ){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    string id_pair = it->first;
    if( os.exist(id_pair) ) 
      val_list.push_back( it->second.get_wnnso() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the WNNSO values
vector<double> oasys_hit_set::get_wnnso_list( ortholog_set& os1, 
					      ortholog_set& os2 ){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    string id_pair = it->first;
    bool flg = false;
    if( !os1.exist(id_pair) and  os2.exist(id_pair) ) flg = true;
    if(  os1.exist(id_pair) and !os2.exist(id_pair) ) flg = false;
    if( flg ) val_list.push_back( it->second.get_wnnso() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return the minimum value of the wnnso values
double oasys_hit_set::get_wnnso_min(){
  map<string, oasys_hit>::iterator it = begin();
  double min = it->second.get_wnnso(); ++it;
  while( it != end() ){
    double score = it->second.get_wnnso();
    if( score < min ) min = score;
    ++it;
  }
  return min;
}


// *************************************************************************
// Class oasys_hit_set
// Return the maximum value of the wnnso values
double oasys_hit_set::get_wnnso_max(){
  map<string, oasys_hit>::iterator it = begin();
  double max = it->second.get_wnnso(); ++it;
  while( it != end() ){
    double score = it->second.get_wnnso();
    if( score > max ) max = score;
    ++it;
  }
  return max;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the bit scores
vector<double> oasys_hit_set::get_bit_list(){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    val_list.push_back( it->second.get_bit() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return the minimum value of the bit scores
double oasys_hit_set::get_bit_min(){
  map<string, oasys_hit>::iterator it = begin();
  double min = it->second.get_bit(); ++it;
  while( it != end() ){
    double score = it->second.get_bit();
    if( score < min ) min = score;
    ++it;
  }
  return min;
}


// *************************************************************************
// Class oasys_hit_set
// Return the maximum value of the bit scores
double oasys_hit_set::get_bit_max(){
  map<string, oasys_hit>::iterator it = begin();
  double max = it->second.get_bit(); ++it;
  while( it != end() ){
    double score = it->second.get_bit();
    if( score > max ) max = score;
    ++it;
  }
  return max;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the synteny scores
vector<double> oasys_hit_set::get_syn_score_list(){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    val_list.push_back( it->second.get_syn_score() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return the minimum value of the synteny scores
double oasys_hit_set::get_syn_score_min(){
  map<string, oasys_hit>::iterator it = begin();
  double min = it->second.get_syn_score(); ++it;
  while( it != end() ){
    double score = it->second.get_syn_score();
    if( score < min ) min = score;
    ++it;
  }
  return min;
}


// *************************************************************************
// Class oasys_hit_set
// Return the maximum value of the synteny scores
double oasys_hit_set::get_syn_score_max(){
  map<string, oasys_hit>::iterator it = begin();
  double max = it->second.get_syn_score(); ++it;
  while( it != end() ){
    double score = it->second.get_syn_score();
    if( score > max ) max = score;
    ++it;
  }
  return max;
}


// *************************************************************************
// Class oasys_hit_set
// Return the WNNSO value that shows the maximum value of the synteny score
double oasys_hit_set::get_syn_score_peak(){
  map<string, oasys_hit>::iterator it = begin();
  double max  = it->second.get_syn_score(); 
  double peak = it->second.get_wnnso();
  ++it;
  while( it != end() ){
    double score = it->second.get_syn_score();
    if( score > max ){
      max  = score;
      peak = it->second.get_wnnso();
    }
    ++it;
  }
  return peak;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the monotonically increasing synteny scores
vector<double> oasys_hit_set::get_mi_syn_score_list(){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    val_list.push_back( it->second.get_mi_syn_score() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return the minimum value of the monotonically increasing synteny scores
double oasys_hit_set::get_mi_syn_score_min(){
  map<string, oasys_hit>::iterator it = begin();
  double min = it->second.get_mi_syn_score(); ++it;
  while( it != end() ){
    double score = it->second.get_mi_syn_score();
    if( score < min ) min = score;
    ++it;
  }
  return min;
}


// *************************************************************************
// Class oasys_hit_set
// Return the maximum value of the monotonically increasing synteny scores
double oasys_hit_set::get_mi_syn_score_max(){
  map<string, oasys_hit>::iterator it = begin();
  double max = it->second.get_mi_syn_score(); ++it;
  while( it != end() ){
    double score = it->second.get_mi_syn_score();
    if( score > max ) max = score;
    ++it;
  }
  return max;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the sequence scores
vector<double> oasys_hit_set::get_seq_score_list(){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    val_list.push_back( it->second.get_seq_score() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return the minimum value of the sequence scores
double oasys_hit_set::get_seq_score_min(){
  map<string, oasys_hit>::iterator it = begin();
  double min = it->second.get_seq_score(); ++it;
  while( it != end() ){
    double score = it->second.get_seq_score();
    if( score < min ) min = score;
    ++it;
  }
  return min;
}


// *************************************************************************
// Class oasys_hit_set
// Return the maximum value of the sequence scores
double oasys_hit_set::get_seq_score_max(){
  map<string, oasys_hit>::iterator it = begin();
  double max = it->second.get_seq_score(); ++it;
  while( it != end() ){
    double score = it->second.get_seq_score();
    if( score > max ) max = score;
    ++it;
  }
  return max;
}


// *************************************************************************
// Class oasys_hit_set
// Return the bit score that shows the maximum value of the sequence score
double oasys_hit_set::get_seq_score_peak(){
  map<string, oasys_hit>::iterator it = begin();
  double max  = it->second.get_seq_score();
  double peak = it->second.get_bit();
  ++it;
  while( it != end() ){
    double score = it->second.get_seq_score();
    if( score > max ){
      max  = score;
      peak = it->second.get_bit();
    }
    ++it;
  }
  return peak;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the monotonically increasing sequence scores
vector<double> oasys_hit_set::get_mi_seq_score_list(){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    val_list.push_back( it->second.get_mi_seq_score() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return the minimum value of the monotonically increasing sequence scores
double oasys_hit_set::get_mi_seq_score_min(){
  map<string, oasys_hit>::iterator it = begin();
  double min = it->second.get_mi_seq_score(); ++it;
  while( it != end() ){
    double score = it->second.get_mi_seq_score();
    if( score < min ) min = score;
    ++it;
  }
  return min;
}


// *************************************************************************
// Class oasys_hit_set
// Return the maximum value of the monotonically increasing sequence scores
double oasys_hit_set::get_mi_seq_score_max(){
  map<string, oasys_hit>::iterator it = begin();
  double max = it->second.get_mi_seq_score(); ++it;
  while( it != end() ){
    double score = it->second.get_mi_seq_score();
    if( score > max ) max = score;
    ++it;
  }
  return max;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the scores
vector<double> oasys_hit_set::get_score_list(){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    val_list.push_back( it->second.get_score() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return the minimum value of the scores
double oasys_hit_set::get_score_min(){
  map<string, oasys_hit>::iterator it = begin();
  double min = it->second.get_score(); ++it;
  while( it != end() ){
    double score = it->second.get_score();
    if( score < min ) min = score;
    ++it;
  }
  return min;
}


// *************************************************************************
// Class oasys_hit_set
// Return the maximum value of the scores
double oasys_hit_set::get_score_max(){
  map<string, oasys_hit>::iterator it = begin();
  double max = it->second.get_score(); ++it;
  while( it != end() ){
    double score = it->second.get_score();
    if( score > max ) max = score;
    ++it;
  }
  return max;
}


// *************************************************************************
// Class oasys_hit_set
// Return a list of the monotonically increasing scores
vector<double> oasys_hit_set::get_mi_score_list(){
  vector<double> val_list;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    val_list.push_back( it->second.get_mi_score() );
    ++it;
  }
  return val_list;
}


// *************************************************************************
// Class oasys_hit_set
// Return the minimum value of the monotonically increasing scores
double oasys_hit_set::get_mi_score_min(){
  map<string, oasys_hit>::iterator it = begin();
  double min = it->second.get_mi_score(); ++it;
  while( it != end() ){
    double score = it->second.get_mi_score();
    if( score < min ) min = score;
    ++it;
  }
  return min;
}


// *************************************************************************
// Class oasys_hit_set
// Return the maximum value of the monotonically increasing scores
double oasys_hit_set::get_mi_score_max(){
  map<string, oasys_hit>::iterator it = begin();
  double max = it->second.get_mi_score(); ++it;
  while( it != end() ){
    double score = it->second.get_mi_score();
    if( score > max ) max = score;
    ++it;
  }
  return max;
}


// *************************************************************************
// Class oasys_hit_set
// Return the hit set
hit_set oasys_hit_set::get_wnnso_hit_set( bool flg ){
  hit_set hs;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    string id_pair = it->first;
    double wnnso   = it->second.get_wnnso();
    if( !flg ){
      list<string> IDs = split(id_pair, " ");
      list<string>::iterator id_it = IDs.begin();
      string id_1 = *id_it; ++id_it;
      string id_2 = *id_it; ++id_it;
      id_pair     = id_2 + " " + id_1;
    }
    hs.insert( id_pair, wnnso );
    ++it;
  }
  return hs;
}


// *************************************************************************
// Class oasys_hit_set
// Return the hit set
hit_set oasys_hit_set::get_mi_score_hit_set( bool flg ){
  hit_set hs;
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    string id_pair  = it->first;
    double mi_score = it->second.get_mi_score();
    if( !flg ){
      list<string> IDs = split(id_pair, " ");
      list<string>::iterator id_it = IDs.begin();
      string id_1 = *id_it; ++id_it;
      string id_2 = *id_it; ++id_it;
      id_pair     = id_2 + " " + id_1;
    }
    hs.insert( id_pair, mi_score );
    ++it;
  }
  return hs;
}


// *************************************************************************
// Class oasys_hit_set
// Return the ortholog set defined by wnnso RBH
ortholog_set oasys_hit_set::get_wnnso_rbh(){
  hit_set hs_12 = get_wnnso_hit_set( true );
  hit_set hs_21 = get_wnnso_hit_set( false );
  c_hit_set c_hs( hs_12, hs_21 );
  top_hit_set t_hs_12( c_hs, true );
  top_hit_set t_hs_21( c_hs, false );
  ortholog_set os = calc_rbh( t_hs_12, t_hs_21 );
  return os;
}


// *************************************************************************
// Class oasys_hit_set
// Return the ortholog set on the basis of the OASYS score
ortholog_set oasys_hit_set::get_oasys_rbh(){
  hit_set hs_12 = get_mi_score_hit_set( true );
  hit_set hs_21 = get_mi_score_hit_set( false );
  c_hit_set c_hs( hs_12, hs_21 );
  top_hit_set t_hs_12( c_hs, true );
  top_hit_set t_hs_21( c_hs, false );
  ortholog_set os = calc_rbh( t_hs_12, t_hs_21 );
  return os;
}


// *************************************************************************
// Class oasys_hit_set
// Calculate synteny scores
void oasys_hit_set::calc_syn_score( model& m1, model& m0 ){
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    it->second.calc_syn_score(m1, m0);
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Calculate monotonically increasing synteny scores
void oasys_hit_set::calc_mi_syn_score( model& m1, model& m0 ){
  double peak_val = get_syn_score_peak();
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    it->second.calc_mi_syn_score(m1, m0, peak_val);
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Calculate sequence scores
void oasys_hit_set::calc_seq_score( model& m1, model& m0 ){
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    it->second.calc_seq_score( m1, m0 );
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Calculate monotonically increasing sequence scores
void oasys_hit_set::calc_mi_seq_score( model& m1, model& m0 ){
  double peak_val = get_seq_score_peak();
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    it->second.calc_mi_seq_score( m1, m0, peak_val );
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Calculate scores which take into account both gene order conservation 
//  and protein sequence conservation
void oasys_hit_set::set_score( double w_ratio ){
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    it->second.set_score( w_ratio );
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Calculate monotonically increasing scores 
//  which take into account both gene order conservation 
//  and protein sequence conservation
void oasys_hit_set::set_mi_score( double w_ratio ){
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    it->second.set_mi_score( w_ratio );
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Print information about this class
void oasys_hit_set::print( string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    string id_pair  = it->first;
    double bit      = it->second.get_bit();
    double wnnso    = it->second.get_wnnso();
    double mi_score = it->second.get_mi_score();
    ofs << id_pair  << "\t" 
	<< bit      << "\t"
	<< wnnso    << "\t"
	<< mi_score << endl;
    ++it;
  }
  ofs.close();
}


// *************************************************************************
// Class oasys_hit_set
// Print bit scores
void oasys_hit_set::print_bit( string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    double bit = it->second.get_bit();
    ofs << bit << endl;
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Print bit score
void oasys_hit_set::print_bit( ortholog_set& os, string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    string id_pair = it->first;
    if( os.exist( id_pair ) ){
      double bit = it->second.get_bit();
      ofs << bit << endl;
    }
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Print wnnso values 
void oasys_hit_set::print_wnnso( string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    double wnnso = it->second.get_wnnso();
    ofs << wnnso << endl;
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Print wnnso values 
void oasys_hit_set::print_wnnso( ortholog_set& os, string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    string id_pair = it->first;
//     if( os.exist( id_pair ) ){
//       double wnnso = it->second.get_wnnso();
//       ofs << wnnso << endl;
//     }
    // for test - start
    double wnnso = it->second.get_wnnso();
    ofs << wnnso << "\t";
    if( os.exist( id_pair ) ){
      ofs << "1";
    } else {
      ofs << "0";
    }
    ofs << endl;
    // for test - end
    ++it;
  }
}


// *************************************************************************
// Class oasys_hit_set
// Print integrated scores
void oasys_hit_set::print_i_score( gene_set& gs1, gene_set& gs2, 
				   string outfile_1, string outfile_2 ){
  ofstream ofs_1( outfile_1.c_str() );
  ofstream ofs_2( outfile_2.c_str() );
  map<string, oasys_hit>::iterator it = begin();
  while( it != end() ){
    string id_pair = it->first;
    double score = it->second.get_mi_score();
    list<string> id_list = split( id_pair, " " );
    list<string>::iterator id_it = id_list.begin();
    string id_1  = *id_it; ++id_it;
    string id_2  = *id_it; ++id_it;
    string str_1 = gs1.get_str( id_1, ":" );
    string str_2 = gs2.get_str( id_2, ":" );
    ofs_1 << str_1 << "\t" << str_2 << "\t" << score << endl;
    ofs_2 << str_2 << "\t" << str_1 << "\t" << score << endl;
    ++it;
  }
  ofs_1.close();
  ofs_2.close();
}


// *************************************************************************
// Class oasys_hit_set
// Draw histogram of synteny scores
void oasys_hit_set::draw_syn_score_histogram( string name ){
  vector<double> val_list = get_syn_score_list();
  histogram h(HIST_BIN_OASYS_HIT_SET, val_list);
  h.draw(name+".syn_score");
}


// *************************************************************************
// Class oasys_hit_set
// Draw histogram of monotonically increasing synteny scores
void oasys_hit_set::draw_mi_syn_score_histogram( string name ){
  vector<double> val_list = get_mi_syn_score_list();
  histogram h(HIST_BIN_OASYS_HIT_SET, val_list);
  h.draw(name+".mi_syn_score");
}


// *************************************************************************
// Class oasys_hit_set
// Draw histogram of sequence scores
void oasys_hit_set::draw_seq_score_histogram( string name ){
  vector<double> val_list = get_seq_score_list();
  histogram h(HIST_BIN_OASYS_HIT_SET, val_list);
  h.draw( name+".seq_score" );
}


// *************************************************************************
// Class oasys_hit_set
// Draw histogram of sequence scores
void oasys_hit_set::draw_mi_seq_score_histogram( string name ){
  vector<double> val_list = get_mi_seq_score_list();
  histogram h(HIST_BIN_OASYS_HIT_SET, val_list);
  h.draw( name+".mi_seq_score" );
}


// *************************************************************************
// Class oasys_hit_set
// Draw histogram of synteny scores
void oasys_hit_set::draw_score_histogram( string name ){
  vector<double> val_list = get_score_list();
  histogram h(HIST_BIN_OASYS_HIT_SET, val_list);
  h.draw(name+".score");
}


// *************************************************************************
// Class oasys_hit_set
// Draw histogram of the monotonically increasing synteny scores
void oasys_hit_set::draw_mi_score_histogram( string name ){
  vector<double> val_list = get_mi_score_list();
  histogram h(HIST_BIN_OASYS_HIT_SET, val_list);
  h.draw(name+".mi_score");
}


// *************************************************************************
// Class oasys_hit_set
// Draw score functions
void oasys_hit_set::draw_score_functions( string name, model& w1, model& w0, 
					  model& b1, model& b0 ){
  double wnnso_min  = get_wnnso_min();
  double wnnso_max  = get_wnnso_max();
  double wnnso_peak = get_syn_score_peak();
  double bit_min    = get_bit_min();
  double bit_max    = get_bit_max();
  double bit_peak   = get_seq_score_peak();
  draw_score_function( name+".syn", w1, w0, wnnso_min, wnnso_max );
  draw_score_function( name+".seq", b1, b0, bit_min, bit_max );
  draw_mi_score_function( name+".mi_syn", w1, w0,
			  wnnso_min, wnnso_max, wnnso_peak );
  draw_mi_score_function( name+".mi_seq", b1, b0, bit_min, bit_max, bit_peak );
}
			   
