// **************************************************************************
//  ogg.cpp
// **************************************************************************

#include "ogg.h"

// **************************************************************************
// Class param_ogg
// Initialize parameter values
param_ogg::param_ogg( vector<double>& x ){
  pdf_ogg ogg;
  ogg.set_param(this);
  ogg.fit_ml(x);
  cerr << "log L  = " << ogg.log_L( x )  << endl
       << "log dL = " << ogg.log_dL( x ) << endl;
  
}


// **************************************************************************
// Class param_ogg
// Initialize parameter values
param_ogg::param_ogg( vector<double>& x, vector<double>& w ){
  pdf_ogg ogg;
  ogg.set_param(this);
  ogg.fit_ml(x, w);
  cerr << "log L  = " << ogg.log_L( x, w )  << endl
       << "log dL = " << ogg.log_dL( x, w ) << endl;
  
}


// **************************************************************************
// Class param_ogg
// Show parameter values
void param_ogg::show(){
  cerr << "Location = " << loc << endl
       << "SD       = " << sd  << endl
       << "Shape    = " << shape << endl;
}


// **************************************************************************
// Class pdf_ogg
// Fit the observed data to this PDF
void pdf_ogg::fit_ml( vector<double>& x ){
  p->loc   = cal_minimum( x );
  p->sd    = cal_sd( x );
  if( p->sd < MIN_SD_OGG ) p->sd = MIN_SD_OGG;
  p->shape = cal_shape( x );
}


// **************************************************************************
// Class pdf_ogg
// Fit the observed data to this PDF
void pdf_ogg::fit_ml( vector<double>& x, vector<double>& w ){
  p->loc   = cal_minimum( x );
  p->sd    = cal_sd( x, w );
  if( p->sd < MIN_SD_OGG ) p->sd = MIN_SD_OGG;
  p->shape = cal_shape( x, w );
}


// **************************************************************************
// Class pdf_ogg
// Print sample points
void pdf_ogg::print_sample_points( string file, double min, double max ){
  ofstream ofs( file.c_str() );
  double width = (max-min)/(PLOT_SAMPLES_OGG-1);
  for( double x=min ; x<max ; x+=width ){
    ofs << x << "\t" << eval(x) << endl;
  }
  ofs << max << "\t" << eval(max) << endl;
  ofs.close();
}


// **************************************************************************
// Class pdf_ogg
// Draw this PDF via GNUplot
void pdf_ogg::plot( string name ){
  string gnufile = name + ".ogg.gnu";
  string epsfile = name + ".ogg.eps";
  string datfile = name + ".ogg.plot.dat";
  double x_max   = p->loc + 3 * p->sd;
  double x_min   = p->loc;
  print_sample_points( datfile, x_min, x_max );
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript"                                      << endl
      << "set output \"" << epsfile << "\""                             << endl
      << "set xrange [" << x_min << ":" << x_max << "]"                 << endl
      << "plot \"" << datfile << "\" with lines "
      << "title \"One-tailed Generalized Gaussian PDF "
      << "(location=" << p->loc << ", sd=" << p->sd
      << ", shape=" << p->shape << ")\" linewidth 2"
      << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
}


// **************************************************************************
// Class pdf_ogg
// Draw this PDF with a histogram via GNUplot
void pdf_ogg::plot( string name, vector<double>& x, size_t num_bin ){
  histogram h( num_bin, x );
  plot( name, h );
}


// **************************************************************************
// Class pdf_ogg
// Draw this PDF with a histogram via GNUplot
void pdf_ogg::plot( string name, vector<double>& x, 
		    vector<double>& w, size_t num_bin ) {
  histogram h( num_bin, x, w );
  plot( name, h );
}


// **************************************************************************
// Class pdf_ogg
// Draw this PDF with a histogram via GNUplot
void pdf_ogg::plot( string name, histogram& h ){
  string datfile = name + ".ogg.hist.dat";
  string pd_file = name + ".ogg.plot.dat";
  string gnufile = name + ".ogg.gnu";
  string epsfile = name + ".ogg.eps";
  double x_min   = h.get_min();
  double x_max   = h.get_max();
  double y1max   = h.get_max_val()*1.1;
  double y2max   = eval(p->loc)*1.1;
  if( p->shape == MIN_SHAPE_OGG ){
    double width = (x_max-x_min)/(NUM_HIST_BIN_OGG*20);
    y2max  = eval(p->loc+width)*1.1;
  }
  h.print( datfile );
  print_sample_points( pd_file, x_min, x_max );
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript color"                                << endl
      << "set output \"" << epsfile << "\""                             << endl
      << "set xrange  [" << x_min << ":" << x_max << "]"                << endl
      << "set yrange  [" << 0     << ":" << y1max << "]"                << endl
      << "set y2range [" << 0     << ":" << y2max << "]"                << endl
      << "set xtics nomirror"                                           << endl
      << "set ytics nomirror"                                           << endl
      << "set style line 1 lt 1"                                        << endl
      << "set style line 2 lt 2"                                        << endl
      << "set style fill solid 0.7"                                     << endl      << "plot \"" << datfile << "\" with boxes axis x1y1 "
      << "title \"Histogram of the observed data\" lt 2, "
      << "\"" << pd_file << "\" with lines axis x1y2 "
      << "title \"OGG PDF "
      << "(location=" << p->loc 
      << ", sd=" << p->sd
      << ", shape=" << p->shape << ")\" linewidth 3 lt 1"
      << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
}


// **************************************************************************
// Class pdf_ogg
// Draw this PDF with various sets of parameters
void pdf_ogg::plot( string name, vector<void*>& param_list ){
  string gnufile = name + ".ogg.gnu";
  string epsfile = name + ".ogg.eps";
  param_ogg* cur_param = p;
  double x_min = p->loc;
  double x_max = p->loc + 3*p->sd;
  vector<string> pd_list;
  vector<double> loc_list, sd_list, shape_list;
  for( unsigned i=0 ; i<param_list.size() ; ++i ){
    string pd_file = name + ".ogg.plot." + int2string(i) + ".dat";
    p = (param_ogg*)param_list[i];
    loc_list.push_back( p->loc );
    sd_list.push_back( p->sd );
    shape_list.push_back( p->shape );
    print_sample_points( pd_file, x_min, x_max );
    pd_list.push_back(pd_file);
  }
  ofstream ofs( gnufile.c_str() );
  ofs << "set terminal postscript"                                      << endl
      << "set output \"" << epsfile << "\""                             << endl
      << "set xrange [" << x_min << ":" << x_max << "]"                 << endl
      << "set xtics nomirror"                                           << endl
      << "set ytics nomirror"                                           << endl
      << "plot \"" << pd_list[0] << "\" with lines "
      << "title \"loc=" << loc_list[0] 
      << ",sd="         << sd_list[0]
      << ",shape="      << shape_list[0] << "\" linewidth 2";
  for( unsigned i=1 ; i<pd_list.size() ; ++i ){
    ofs << ", \"" << pd_list[i] << "\" with lines "
	<< "title \"loc=" << loc_list[i] 
	<< ",sd="         << sd_list[i]
	<< ",shape="      << shape_list[i] << "\" linewidth 2";
  }
  ofs << endl;
  ofs.close();
  string cmd = "gnuplot " + gnufile;
  system( cmd.c_str() );
  p = cur_param;
}


// **************************************************************************
// Class pdf_ogg
// Evaluate this PDF
double pdf_ogg::eval( double x ){
  if( x < p->loc ) return 0;
  double gamma = 1.0/p->sd*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  return p->shape*gamma/gsl_sf_gamma(1.0/p->shape)*exp(-1.0*pow(gamma,p->shape)*pow(x-p->loc,p->shape));
}


// **************************************************************************
// Class pdf_ogg
// Evaluate the log-likelihood function
double pdf_ogg::log_L( vector<double>& x ){
  size_t N = x.size();
  double gamma = 1.0/p->sd*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double log_L = (double)N*log(gamma*p->shape/gsl_sf_gamma(1.0/p->shape));
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( x[i]-p->loc > NEARLY_ZERO_OGG ){
      log_L -= pow((x[i]-p->loc)*gamma, p->shape);
    }
  }
  return log_L;
}


// **************************************************************************
// Class pdf_ogg
// Evaluate the log-likelihood function
double pdf_ogg::log_L( vector<double>& x, vector<double>& w ){
  if( x.size() != w.size() ){
    cerr << "Error! the number of the data points does not equal to "
	 << "the number of weights." << endl;
    exit(-1);
  }
  double cnt = 0.0;
  for( unsigned i=0 ; i<w.size() ; ++i ){
    cnt += w[i];
  }

  double gamma = 1.0/p->sd*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  double log_L = (double)cnt*log(gamma*p->shape/gsl_sf_gamma(1.0/p->shape));
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( x[i]-p->loc > NEARLY_ZERO_OGG ){
      log_L -= w[i] * pow((x[i]-p->loc)*gamma, p->shape);
    }
  }
  return log_L;
}


// **************************************************************************
// Class pdf_ogg
// Evaluate the derivative of the log-likelihood function
double pdf_ogg::log_dL( vector<double>& x ){
  size_t N = x.size();
  double c = p->shape;
  double D2 = 0.5*(2.0/c-3.0*gsl_sf_psi(3.0/c)/pow(c,2)+3*gsl_sf_psi(1.0/c)/pow(c,2));
  double g_ratio = (gsl_sf_psi(1.0/c) - 3.0*gsl_sf_psi(3.0/c)) / (2*pow(c,2));
  double log_dL = N*D2;
  double gamma = 1.0/p->sd*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( x[i]-p->loc > NEARLY_ZERO_OGG ){
      log_dL -= pow( (x[i]-p->loc)*gamma, p->shape )
	* ( log( (x[i]-p->loc)*gamma ) + c*g_ratio );
    }
  }
  return log_dL;
}


// **************************************************************************
// Class pdf_ogg
// Evaluate the derivative of the log-likelihood function
double pdf_ogg::log_dL( vector<double>& x, vector<double>& w ){
  if( x.size() != w.size() ){
    cerr << "Error! the number of the data points does not equal to "
	 << "the number of weights." << endl;
    exit(-1);
  }
  double cnt = 0.0;
  for( unsigned i=0 ; i<w.size() ; ++i ){
    cnt += w[i];
  }
  double c = p->shape;
  double D2 = 0.5*(2.0/c-3.0*gsl_sf_psi(3.0/c)/pow(c,2)+3*gsl_sf_psi(1.0/c)/pow(c,2));
  double g_ratio = (gsl_sf_psi(1.0/c) - 3.0*gsl_sf_psi(3.0/c)) / (2*pow(c,2));
  double log_dL = cnt*D2;
  double gamma = 1.0/p->sd*sqrt(gsl_sf_gamma(3.0/p->shape)/gsl_sf_gamma(1.0/p->shape));
  for( unsigned i=0 ; i<x.size() ; ++i ){
    if( x[i]-p->loc > NEARLY_ZERO_OGG ){
      log_dL -= w[i] * pow( (x[i]-p->loc)*gamma, p->shape )
	* ( log( (x[i]-p->loc)*gamma ) + c*g_ratio );
    }
  }
  return log_dL;
}


// **************************************************************************
// Class pdf_ogg
// Return the sample standard deviation
double pdf_ogg::cal_sd( vector<double>& x ){
  double var = 0.0;
  for( unsigned i=0 ; i<x.size() ; ++i ){
    var += pow(x[i]-p->loc,2);
  }
  return sqrt(var/(x.size()-1));
}


// **************************************************************************
// Class pdf_ogg
// Return the minimum value
double pdf_ogg::cal_minimum( vector<double>& x ){
  double min = x[0];
  for( unsigned i=1 ; i<x.size() ; ++i ){
    if( x[i]<min ) min=x[i];
  }
  return min;
}


// **************************************************************************
// Class pdf_ogg
// Return the sample standard deviation
double pdf_ogg::cal_sd( vector<double>& x, vector<double>& w ){
  if( x.size() != w.size() ){
    cerr << "Error! the number of the data points does not equal to "
	 << "the number of weights." << endl;
    exit(-1);
  }

  double var = 0.0;
  double cnt = 0.0;
  for( unsigned i=0 ; i<x.size() ; ++i ){
    cnt += w[i];
    var += w[i] * pow(x[i]-p->loc,2);
  }
  return sqrt(var/(cnt-1.0));
}


// **************************************************************************
// Class pdf_ogg
// Return the local optimum for the shape parameter
// Computation is based on the FR method
double pdf_ogg::cal_shape( vector<double>& x ){
  p->shape = MIN_SHAPE_OGG;
  if( log_dL( x ) < 0 )
    return MIN_SHAPE_OGG;

  const gsl_multimin_fdfminimizer_type* T 
    = gsl_multimin_fdfminimizer_conjugate_fr;
  // = gsl_multimin_fdfminimizer_vector_bfgs;
  gsl_multimin_fdfminimizer* s
    = gsl_multimin_fdfminimizer_alloc( T, 1 );

  gsl_multimin_function_fdf obj_func;
  gsl_param_ogg gp;
  gp.first        = &x;
  gp.second       = p;
  obj_func.f      = &ogg_f;
  obj_func.df     = &ogg_df;
  obj_func.fdf    = &ogg_fdf;
  obj_func.n      = 1;
  obj_func.params = &gp;
  gsl_vector* v = gsl_vector_alloc(1); 
  gsl_vector_set(v, 0, p->shape);

  size_t iter = 0;
  gsl_multimin_fdfminimizer_set( s, &obj_func, v, STEP_SIZE_OGG, TOL_OGG );
  int status;
  do {
    ++iter;
    status = gsl_multimin_fdfminimizer_iterate(s);
    if( status ) break;
    status = gsl_multimin_test_gradient(s->gradient, EPSABS_OGG);
    double shape = gsl_vector_get(s->x,0);
    cerr << iter << "-th iteration (shape = " << shape << ")" << endl;
  } while ( status == GSL_CONTINUE && iter < ITER_OGG );
  double shape = gsl_vector_get(s->x, 0);
  cerr << iter << " steps." << endl;
  gsl_multimin_fdfminimizer_free(s);
  gsl_vector_free(v);

  return shape;
}


// **************************************************************************
// Class pdf_ogg
// Return the local optimum for the shape parameter
// Computation is based on the FR method
double pdf_ogg::cal_shape( vector<double>& x, vector<double>& w ){
  p->shape = MIN_SHAPE_OGG;
  if( log_dL( x, w ) < 0 )
    return MIN_SHAPE_OGG;

  const gsl_multimin_fdfminimizer_type* T 
    = gsl_multimin_fdfminimizer_conjugate_fr;
  // = gsl_multimin_fdfminimizer_vector_bfgs;
  gsl_multimin_fdfminimizer* s
    = gsl_multimin_fdfminimizer_alloc( T, 1 );

  gsl_multimin_function_fdf obj_func;
  gsl_param_w_ogg w_gp;
  w_gp.push_back(&x);
  w_gp.push_back(&w);
  w_gp.push_back(p);
  obj_func.f      = &w_ogg_f;
  obj_func.df     = &w_ogg_df;
  obj_func.fdf    = &w_ogg_fdf;
  obj_func.n      = 1;
  obj_func.params = &w_gp;
  gsl_vector* v = gsl_vector_alloc(1); 
  gsl_vector_set(v, 0, p->shape);

  size_t iter = 0;
  gsl_multimin_fdfminimizer_set( s, &obj_func, v, STEP_SIZE_OGG, TOL_OGG );
  int status;
  do {
    ++iter;
    status = gsl_multimin_fdfminimizer_iterate(s);
    if( status ) break;
    status = gsl_multimin_test_gradient(s->gradient, EPSABS_OGG);
    double shape = gsl_vector_get(s->x,0);
    cerr << iter << "-th iteration (shape = " << shape << ")" << endl;
  } while ( status == GSL_CONTINUE && iter < ITER_OGG );
  double shape = gsl_vector_get(s->x, 0);
  cerr << iter << " steps." << endl;
  gsl_multimin_fdfminimizer_free(s);
  gsl_vector_free(v);

  return shape;
}


// **************************************************************************
// Class pdf_ogg
// Function needed to perform the optimization of the shape parameter (f)
double ogg_f( const gsl_vector* v, void* gsl_param ){
  gsl_param_ogg    gp   = *(gsl_param_ogg*)gsl_param;
  vector<double>* data  = gp.first;
  param_ogg*      p_ogg = gp.second;
  double shape = gsl_vector_get(v, 0);
  p_ogg->set_shape( shape );
  pdf_ogg ogg;
  ogg.set_param( p_ogg );
  return -1.0 * ogg.log_L( *data );
};


// **************************************************************************
// Class pdf_ogg
// Function needed to perform the optimization of the shape parameter (df)
void ogg_df( const gsl_vector* v, void* gsl_param, gsl_vector* df ){
  gsl_param_ogg    gp   = *(gsl_param_ogg*)gsl_param;
  vector<double>* data  = gp.first;
  param_ogg*      p_ogg = gp.second;
  double shape = gsl_vector_get(v, 0);
  p_ogg->set_shape( shape );
  pdf_ogg ogg;
  ogg.set_param( p_ogg );
  double log_dL = ogg.log_dL( *data );
  gsl_vector_set( df, 0, -1.0*log_dL );
}


// **************************************************************************
// Class pdf_ogg
// Function needed to perform the optimization of the shape parameter (fdf)
void ogg_fdf( const gsl_vector* v, void* data, double* f, gsl_vector* df ){
  *f = ogg_f( v, data );
  ogg_df( v, data, df );
}


// **************************************************************************
// Class pdf_ogg
// Function needed to perform the optimization of the shape parameter (f)
double w_ogg_f( const gsl_vector* v, void* gsl_param ){
  gsl_param_w_ogg w_gp   = *(gsl_param_w_ogg*)gsl_param;
  vector<double>* data   = (vector<double>*)w_gp[0];
  vector<double>* weight = (vector<double>*)w_gp[1];
  param_ogg*      p_ogg  = (param_ogg*)w_gp[2];
  double shape = gsl_vector_get(v, 0);
  p_ogg->set_shape( shape );
  pdf_ogg ogg;
  ogg.set_param( p_ogg );
  return -1.0 * ogg.log_L( *data, *weight );
};


// **************************************************************************
// Class pdf_ogg
// Function needed to perform the optimization of the shape parameter (df)
void w_ogg_df( const gsl_vector* v, void* gsl_param, gsl_vector* df ){
  gsl_param_w_ogg w_gp   = *(gsl_param_w_ogg*)gsl_param;
  vector<double>* data   = (vector<double>*)w_gp[0];
  vector<double>* weight = (vector<double>*)w_gp[1];
  param_ogg*      p_ogg  = (param_ogg*)w_gp[2];
  double shape = gsl_vector_get(v, 0);
  p_ogg->set_shape( shape );
  pdf_ogg ogg;
  ogg.set_param( p_ogg );
  double log_dL = ogg.log_dL( *data, *weight );
  gsl_vector_set( df, 0, -1.0*log_dL );
}


// **************************************************************************
// Class pdf_ogg
// Function needed to perform the optimization of the shape parameter (fdf)
void w_ogg_fdf( const gsl_vector* v, void* data, double* f, gsl_vector* df ){
  *f = w_ogg_f( v, data );
  w_ogg_df( v, data, df );
}
