// *************************************************************************
//  options.cpp
// *************************************************************************

#include "options.h"
using namespace std;

// *************************************************************************
// Class options
// Constructor
// Parse command line and get options
options::options( int argc, char* argv[] ){
  char opt;
  int opt_idx;
  bool flg_I = false;
  bool flg_J = false;
  bool flg_P = false;
  bool flg_G = false;
  bool flg_O = false;
  bool flg_H = false;

  // set default parameters
  col_index = DEFAULT_COL_INDEX;
  sds       = DEFAULT_SDS;
  t_dpd     = DEFAULT_T_DPD;
  w_ratio   = DEFAULT_W_RATIO; 
  flg_short_cut = false;
  flg_plot      = false;
  flg_group     = false;

  string usage;
  usage = (string)"\n"
    + "*************************************************************"  + "\n"
    + "Description:"                                                   + "\n"
    + "  OASYS -- Ortholog Assignment using SYnteny"                   + "\n"
    + "          and Sequence imformation"                             + "\n"
                                                                       + "\n"
    + "Options:"                                                       + "\n"
    + "  -i  set organism name (1)"                                    + "\n"
    + "  -j  set organism name (2)"                                    + "\n"
    + "  -m  set file containing many-to-many orthology assignents"    + "\n"
    + "  -p  set path to directory containing oasys_in files"          + "\n"
    + "  -g  set path to directory containing gene files"              + "\n"
    + "  -o  set output directory"                                     + "\n"
    + "  -l  set collinearity index"                                   + "\n"
    + "       (default = " + double2string( DEFAULT_COL_INDEX ) + ")"  + "\n"
    + "  -s  set sigma parameter"                                      + "\n"
    + "       (default = " + double2string( DEFAULT_SDS ) + ")"        + "\n"
    + "  -t  set threshold for DPD"                                    + "\n"
    + "       (default = " + int2string( DEFAULT_T_DPD ) + ")"         + "\n"
    + "  -w  set weight ratio (w_syn / w_seq)"                         + "\n"
    + "       (default = " + double2string( DEFAULT_W_RATIO ) + ")"    + "\n"
    + "  -c  turn on the short cut option"                             + "\n"
    + "  -P  turn on the plot option"                                  + "\n"
    + "  -h  print this message"                                       + "\n"
    + "*************************************************************"  + "\n";
    
  opt_idx = 0;
  while( -1 ){
    opt = getopt( argc, argv, "i:j:m:p:g:o:l:s:d:w:cPh" );
    opt_idx++;
    if( opt == -1 ){
      break;
    }

    switch( opt ){
    case 'i':
      orgname_in1 = (string)argv[++opt_idx];
      flg_I = true;
      break;
    case 'j':
      orgname_in2 = (string)argv[++opt_idx];
      flg_J = true;
      break;
    case 'm':
      group_file = (string)argv[++opt_idx];
      flg_group = true;
      if( flg_P ) flg_P = false;
      else        flg_P = true;
      break;
    case 'p':
      data_dir = (string)argv[++opt_idx];
      if( flg_P ) flg_P = false;
      else        flg_P = true;
      break;
    case 'g':
      gene_dir = (string)argv[++opt_idx];
      flg_G = true;
      break;
    case 'o':
      out_dir = (string)argv[++opt_idx];
      flg_O = true;
      break;
    case 'l':
      col_index = atof( argv[++opt_idx] );
      break;
    case 's':
      sds = atof( argv[++opt_idx] );
      break;
    case 'd':
      t_dpd = atoi( argv[++opt_idx] );
      break;
    case 'w':
      w_ratio = atof( argv[++opt_idx] );
      break;
    case 'c':
      flg_short_cut = true;
      break;
    case 'P':
      flg_plot = true;
      break;
    case 'h':
      flg_H = true;
      break;
    case '?':
    default:
      cerr << "Unknown option " << opt << endl;
      exit( -1 );
    }
  }
  if( flg_H || !flg_I || !flg_J || !flg_P || !flg_G || !flg_O ){
    cerr << usage << endl;
    exit(1);
  }
  set_file_name();
}


// *************************************************************************
// Class options
// Set file names
void options::set_file_name() {
  hitfile_12 
    = data_dir + "/" + orgname_in1 + ".vs." + orgname_in2 + ".oasys_in";
  hitfile_21
    = data_dir + "/" + orgname_in2 + ".vs." + orgname_in1 + ".oasys_in";
  jobname 
    = out_dir + "/" + orgname_in1 + ".vs." + orgname_in2;
  opt_file          = jobname + ".options";
  genefile_1        = gene_dir + "/" + orgname_in1 + ".genes";
  genefile_2        = gene_dir + "/" + orgname_in2 + ".genes";
  wnnso_file        = jobname + ".wnnso_values";
  oasys_hit_file    = jobname + ".hits";
  bit_rbh_file      = jobname + ".bit.rbh";
  wnnso_rbh_file    = jobname + ".wnnso.rbh";
  oasys_rbh_file    = jobname + ".oasys.rbh";
  ortholog_file     = jobname + ".oasys.orthologs";
}


// *************************************************************************
// Class options
// Print options
void options::print() {
  string options = (string)"\n"
    + "********************************************************"     + "\n"
    + "Options:"                                                     + "\n"
    + "IN organism (1)        : " + orgname_in1                      + "\n"
    + "IN organism (2)        : " + orgname_in2                      + "\n"
    + "Group file             : "; 
  if( flg_group )
    options += group_file;
  else 
    options += (string)"Not specified";
  options += (string)"\n"
    + "Data directory         : " + data_dir                         + "\n"
    + "Gene directory         : " + gene_dir                         + "\n"
    + "Hit file (1 -> 2)      : " + hitfile_12                       + "\n"
    + "Hit file (2 -> 1)      : " + hitfile_21                       + "\n"
    + "Collinearity index     : " + double2string( col_index )       + "\n"
    + "Sigma parameter        : " + double2string( sds )             + "\n"
    + "Threshold for DPD      : " + int2string( t_dpd )              + "\n"
    + "Weight ratio           : " + double2string( w_ratio )         + "\n"
    + "Short cut option       : " + bool2string( flg_short_cut )     + "\n"
    + "Plot option            : " + bool2string( flg_plot )          + "\n"
    + "Output directory       : " + out_dir                          + "\n"
    + "********************************************************"     + "\n";
  cerr << options << endl;
}


// *************************************************************************
// Class options
// Print options
void options::print( string outfile ) {
  ofstream ofs( outfile.c_str() );
  string options = (string)"\n"
    + "********************************************************"     + "\n"
    + "Options:"                                                     + "\n"
    + "IN organism (1)        : " + orgname_in1                      + "\n"
    + "IN organism (2)        : " + orgname_in2                      + "\n"
    + "Group file             : "; 
  if( flg_group )
    options += group_file;
  else 
    options += (string)"Not specified";
  options += (string)"\n"
    + "Data directory         : " + data_dir                         + "\n"
    + "Gene directory         : " + gene_dir                         + "\n"
    + "Hit file (1 -> 2)      : " + hitfile_12                       + "\n"
    + "Hit file (2 -> 1)      : " + hitfile_21                       + "\n"
    + "Collinearity index     : " + double2string( col_index )       + "\n"
    + "Sigma parameter        : " + double2string( sds )             + "\n"
    + "Threshold for DPD      : " + int2string( t_dpd )              + "\n"
    + "Weight ratio           : " + double2string( w_ratio )         + "\n"
    + "Short cut option       : " + bool2string( flg_short_cut )     + "\n"
    + "Plot option            : " + bool2string( flg_plot )          + "\n"
    + "Output directory       : " + out_dir                          + "\n"
    + "********************************************************"     + "\n";
  ofs << options << endl;
  ofs.close();
}
