// *************************************************************************
//  ortholog_set.cpp
// *************************************************************************

#include "ortholog_set.h"
using namespace std;


// *************************************************************************
// Class ortholog_set
// Constructor
ortholog_set::ortholog_set( string in_file ){
  try {
    ifstream ifs( in_file.c_str() );
    if( ifs.fail() )
      throw (string)"File open error: " + in_file.c_str();
    string str;
    while( getline(ifs, str) ){
      list<string> cols = split( str, "\t" );
      if( cols.size() != 3 )
	throw "Inappropriate file format: " + str;
      list<string>::iterator it = cols.begin();
      string id1 = *it; ++it;
      string id2 = *it; ++it;
      double score = atof( (*it).c_str() ); ++it;
      insert( id1, id2, score );
    }
    ifs.close();
  } catch ( string err_msg ) {
    cerr << err_msg << endl; 
    exit(-1);
  }
}


// *************************************************************************
// Class ortholog_set
// Insert new ortholog
void ortholog_set::insert( string ip, double score ){
  gp2score.erase( ip );
  gp2score.insert( map<string, double>::value_type( ip, score ) );
}


// *************************************************************************
// Class ortholog_set
// Return the score of the gene pair specified by ID pair
double ortholog_set::get_score( string ip ){
  map<string, double>::iterator it = find( ip );
  if( it == end() ){
    cerr << "Error! : " << ip << " is not found." << endl;
    exit(-1);
  }
  return it->second;
}


// *************************************************************************
// Class ortholog_set
// Return the set of common hits
c_hit_set ortholog_set::get_c_hit_set(){
  c_hit_set c_hs;
  map<string, double>::iterator it = begin();
  while( it != end() ){
    string ip = it->first;
    double score = it->second;
    c_hit h(ip, score);
    c_hs.insert( ip, h );
    ++it;
  }
  return c_hs;
}


// *************************************************************************
// Class ortholog_set
// Print the list of ID pair
void ortholog_set::print_IDpair( string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, double>::iterator os_it = begin();
  while( os_it != end() ){
    string ip = os_it->first;
    ofs << ip << endl;
    ++os_it;
  }
}


// *************************************************************************
// Class ortholog_set
// Print the list of ID pair
void ortholog_set::print_info( string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, double>::iterator os_it = begin();
  while( os_it != end() ){
    string ip    = os_it->first;
    list<string> IDs = split(ip, " ");
    list<string>::iterator id_it = IDs.begin();
    string id_1 = *id_it; ++id_it;
    string id_2 = *id_it; ++id_it;
    double score = os_it->second;
    ofs << id_1 << "\t" << id_2 << "\t" << score << endl;
    ++os_it;
  }
}


// *************************************************************************
// Class ortholog_set
// Print the anchors
void ortholog_set::print_anchor( gene_set& gs1, gene_set& gs2, 
				 string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, double>::iterator os_it = begin();
  while( os_it != end() ){
    string ip    = os_it->first;
    list<string> IDs = split(ip, " ");
    list<string>::iterator id_it = IDs.begin();
    string id_1  = *id_it; ++id_it;
    string id_2  = *id_it; ++id_it;
    string loc_1 = gs1.get_locus( id_1 );
    string loc_2 = gs2.get_locus( id_2 );
    ofs << loc_1 << " " << loc_2  << endl;
    ++os_it;
  }
}


// *************************************************************************
// Calculate non-seed gene pairs
ortholog_set calc_non_seed_gene_pair( c_hit_set& c_hs, 
				      ortholog_set& seeds ){
  ortholog_set non_seeds;
  map<string, c_hit>::iterator c_it = c_hs.begin();
  while( c_it != c_hs.end() ){
    string ip = c_it->first;
    if( !seeds.exist( ip ) ){
      double score = c_hs.get_bit( ip );
      non_seeds.insert( ip, score );
    }
    ++c_it;
  }
  return non_seeds;
}


// *************************************************************************
// Calculate RBHs
ortholog_set calc_rbh( top_hit_set& t_hs_12, top_hit_set& t_hs_21 ){
  ortholog_set rbh;
  map<string, string>::iterator it_12 = t_hs_12.begin_id();
  while( it_12 != t_hs_12.end_id() ){
    string id_1 = it_12->first;
    string id_2 = it_12->second;
    map<string, string>::iterator it_21 = t_hs_21.find_id( id_2 );
    if( it_21 != t_hs_21.end_id() ){
      if( it_21->second == id_1 ){
	double score = t_hs_12.get_bit( id_1 );
	rbh.insert( id_1, id_2, score );
      }
    }
    ++it_12;
  }
  return rbh;
}


