// *************************************************************************
//  position_map.cpp
// *************************************************************************

#include "position_map.h"
using namespace std;


// *************************************************************************
// Class position_map
// Constructor
position_map::position_map( gene_set& gs ){
  map<string, bool> chroms = get_chroms( gs );
  map<string, bool>::iterator ch_it = chroms.begin();
  while( ch_it != chroms.end() ){
    string   chr    = ch_it->first;
    gene_set tmp_gs = get_gene_set( gs, chr );
    position_list pl( tmp_gs );
    insert( chr, pl );
    ++ch_it;
  }
  set_id2chr( gs );
  set_id2sgn( gs );
}


// *************************************************************************
// Class position_map
// Set the variable id2chr
void position_map::set_id2chr( gene_set& gs ){
  map<string, gene>::iterator gs_it = gs.begin();
  while( gs_it != gs.end() ){
    string id  = gs_it->first;
    gene   g   = gs_it->second;
    string chr = g.get_chr();
    id_insert( id, chr );
    ++gs_it;
  }
}


// *************************************************************************
// Class position_map
// Set the variable id2sgn
void position_map::set_id2sgn( gene_set& gs ){
  map<string, gene>::iterator gs_it = gs.begin();
  while( gs_it != gs.end() ){
    string id  = gs_it->first;
    gene   g   = gs_it->second;
    bool   sgn = g.get_sgn();
    id_insert( id, sgn );
    ++gs_it;
  }
}


// *************************************************************************
// Class position_map
// Return a number of genes included in the position map
unsigned position_map::gene_size(){
  map<string, position_list>::iterator it = begin();
  unsigned gene_size = 0;
  while( it != end() ){
    gene_size += it->second.size();
    ++it;
  }
  return gene_size;
}


// *************************************************************************
// Class position_map
// Return a position list specified by a given chromosome ID
position_list position_map::get_position_list( string chr ){
  map<string, position_list>::iterator it = find( chr );
  if( it == end() ){
    cerr << "Error: " << chr << " is not found." << endl;
    exit( -1 );
  }
  return it->second;
}

// *************************************************************************
// Class position_map
// Return a set of chromosomes 
map<string, bool> position_map::get_chroms(){
  map<string, bool> chroms;
  map<string, position_list>::iterator it = begin();
  while( it != end() ){
    string chr = it->first;
    chroms.insert( map<string, bool>::value_type( chr, true ) );
    ++it;
  }
  return chroms;
}


// *************************************************************************
// Class position_map
// Return a set of chromosomes 
map<string, bool> position_map::get_chroms( gene_set& gs ){
  map<string, bool> chroms;
  map<string, gene>::iterator gs_it = gs.begin();
  while( gs_it != gs.end() ){
    string id = gs_it->first;
    gene   g  = gs_it->second;
    string chrom = g.get_chr();
    map<string, bool>::iterator ch_it = chroms.find( chrom );
    if( ch_it == chroms.end() ){
      chroms.insert( map<string, bool>::value_type( chrom, true ) );
    }
    ++gs_it;
  }
  return chroms;
}


// *************************************************************************
// Class position_map
// Return the chromosome specified by a given gene ID
string position_map::get_chrom( string id ){
  map<string, string>::iterator it = id_find( id );
  if( it == id_end() ){
    cerr << "Error: " << id << " is not found." << endl;
    exit( -1 );
  }
  string chr = it->second;
  return chr;
}


// *************************************************************************
// Class position_map
// Return the sign specified by a given gene ID
bool position_map::get_sign( string id ){
  map<string, bool>::iterator it = id2sgn_find( id );
  if( it == id2sgn_end() ){
    cerr << "Error: " << id << " is not found." << endl;
    exit( -1 );
  }
  bool flg = it->second;
  return flg;
}


// *************************************************************************
// Return a set of genes encoded in a given chromosome
gene_set position_map::get_gene_set( gene_set& gs, string chr ){
  gene_set tmp_gs;
  map<string, gene>::iterator gs_it = gs.begin();
  while( gs_it != gs.end() ){
    string id      = gs_it->first;
    gene   g       = gs_it->second;
    string cur_chr = g.get_chr();
    if( cur_chr == chr ){ tmp_gs.insert( id, g ); }
    ++gs_it;
  }
  return tmp_gs;
}
