// *************************************************************************
//  seed_matrix.cpp
// *************************************************************************

#include "seed_matrix.h"
using namespace std;

// *************************************************************************
// Class seed_matrix
// Constructor
seed_matrix::seed_matrix( string in_file ){
  ifstream ifs( in_file.c_str() );
  string str;
  getline( ifs, str );
  list<string> str_list = split( str, ":" );
  list<string>::iterator str_it = str_list.begin();
  string n_cols_str = *str_it; ++str_it; n_cols = atoi( n_cols_str.c_str() );
  string n_rows_str = *str_it; ++str_it; n_rows = atoi( n_rows_str.c_str() );
  unsigned nr = n_rows;
  while( getline( ifs, str ) ){
    str_list.clear();
    str_list = split( str, " " );
    str_it   = str_list.begin();
    unsigned nc = 0;
    while( str_it != str_list.end() ){
      string sign_str = *str_it;
      int sign;
      if( sign_str == "." ){
	sign = 0;
      } else if( sign_str == "+" ){
	sign = 1;
      } else if( sign_str == "-" ){
	sign = -1;
      } else {
	cerr << "Error! " << sign_str << " is wrong entry." << endl;
	exit( -1 );
      }
      coords c = coords::pair( ++nc, nr );
      insert( c, sign );
      ++str_it;
    }
    if( nc != n_cols ){
      cerr << "Error! " << nc << " columns in the " << nr << "-th row."
	   << endl;
      exit( -1 );
    }
    --nr;
  }
  if( nr != 0 ){
    cerr << "Error! " << (n_rows-nr) << " rows in the input file" << endl;
    exit( -1 );
  }
}


// *************************************************************************
// Class seed_matrix
// Constructor
seed_matrix::seed_matrix( unsigned nc, unsigned nr, string seed_pos ){
  n_cols = nc;
  n_rows = nr;
  initialize();
  insert_seeds( seed_pos );
}


// *************************************************************************
// Class seed_matrix
// Initialize the matrix
void seed_matrix::initialize() {
  for( unsigned i=1 ; i<=n_cols ; ++i ){
    for( unsigned j=1 ; j<=n_rows ; ++j ){
      coords c = coords::pair( i, j );
      insert( c, 0 );
    }
  }
}


// *************************************************************************
// Class seed_matrix
// Calculate the WNNSO values
void seed_matrix::calc_wnnso( double sigma ){
  for( unsigned i=1 ; i<=n_cols ; ++i ){
    for( unsigned j=1 ; j<=n_rows ; ++j ){
      coords c = coords::pair( i, j );
      calc_wnnso( c, sigma );
    }
  }
}


// *************************************************************************
// Class seed_matrix
// Calculate the WNNSO value of the specified element
void seed_matrix::calc_wnnso( coords c, double sigma ){
  double p_wnnso = 0.0;
  double r_wnnso = 0.0;
  map< coords, int >::iterator it = begin();
  while( it != end() ){
    coords cur_c = it->first;
    int sign = it->second;
    if( sign == 1 ){
      p_wnnso += calc_weight( c, cur_c, true, sigma );
    } else if( sign == -1 ){
      r_wnnso += calc_weight( c, cur_c, false, sigma );
    }
    ++it;
  }
  p_insert( c, p_wnnso );
  r_insert( c, r_wnnso );
}


// *************************************************************************
// Class seed_matrix
// Return the weight value between two elements in the matrix
double seed_matrix::calc_weight( coords c1, coords c2, 
				 bool sgn, double sigma ){
  unsigned dpd = calc_dpd( c1, c2, sgn );
  double weight = dpd2weight( dpd, sigma );
  return weight;
}


// *************************************************************************
// Class seed_matrix
// Return the DPD value between two elements in the matrix
unsigned seed_matrix::calc_dpd( coords c1, coords c2, bool sgn ){
  if( is_collinear( c1, c2, sgn ) ){
    unsigned i_1 = c1.first;
    unsigned i_2 = c1.second;
    unsigned j_1 = c2.first;
    unsigned j_2 = c2.second;
    unsigned dpd = 2 * util::max( util::abs( (int)i_1 - (int)j_1 ), 
				  util::abs( (int)i_2 - (int)j_2 ))
      - util::min( util::abs( (int)i_1 - (int)j_1 ), 
		   util::abs( (int)i_2 - (int)j_2 ));
    return dpd;
  } else {
    return INF;
  }
}


// *************************************************************************
// Class seed_matrix
// Return whether given two elements are collinear, or not
bool seed_matrix::is_collinear( coords c1, coords c2, bool sgn ){
    unsigned i_1 = c1.first;
    unsigned i_2 = c1.second;
    unsigned j_1 = c2.first;
    unsigned j_2 = c2.second;
    if( i_1 == j_1 ) { return false; }
    if( i_2 == j_2 ) { return false; }
    double slope = (double)( (int)i_2 - (int)j_2 ) / ( (int)i_1 - (int)j_1 );
    if( sgn ) { return slope>0 ? true : false; }
    else { return slope<0 ? true : false; }
}


// *************************************************************************
// Class seed_matrix
// Return the weight value
double seed_matrix::dpd2weight( unsigned dpd, double sigma ){
  if( dpd == INF )
    return 0.0;
  else 
    return 1 / ( sigma * sqrt( 2 * M_PI ) ) 
    * exp( -1 * (double)dpd / ( 2 * sigma * sigma ) );
}


// *************************************************************************
// Class seed_matrix
// Insert seeds
void seed_matrix::insert_seeds( string seed_pos ) {
  // Detele the first and last characters
  seed_pos = seed_pos.substr(1, seed_pos.length()-2 ); 
  list<string> pos_list = split( seed_pos, "][" );
  list<string>::iterator pos_it = pos_list.begin();
  while( pos_it != pos_list.end() ){
    string pos = *pos_it;
    list<string> pos_info = split( pos, "," );
    list<string>::iterator info_it = pos_info.begin();
    string i_str = *info_it; ++info_it; unsigned i = atoi( i_str.c_str() );
    string j_str = *info_it; ++info_it; unsigned j = atoi( j_str.c_str() );
    string s_str = *info_it; ++info_it; int      s = atoi( s_str.c_str() );
    coords coords = coords::pair( i, j );
    erase( coords );
    insert( coords, s );
    ++pos_it;
  }
}


// *************************************************************************
// Class seed_matrix
// Return the number of seeds in the matrix
unsigned seed_matrix::num_seeds() {
  unsigned num_seeds = 0;
  map< coords, int >::iterator it = begin();
  while( it != end() ){
    int sign = it->second;
    if( sign ) ++num_seeds;
    ++it;
  }
  return num_seeds;
}


// *************************************************************************
// Class seed_matrix
// Print the matrix
void seed_matrix::show(){
  cerr << n_cols << ":" << n_rows << endl;
  for( unsigned i=n_cols ; ; --i ){
    for( unsigned j=1 ; j<=n_rows ; ++j ){
      coords c = coords::pair( j, i );
      map< coords, int >::iterator it = find( c );
      if( it == end() ){
	cerr << "Error! (" << i << "," << j << ") is not found." << endl;
	exit(-1);
      }
      int sign = it->second;
      if( j!=1 ) cerr << " ";
      if( sign == -1 ){
	cerr << '-';
      } else if( sign == 1 ){
	cerr << '+';
      } else {
	cerr << '.';
      }
    }
    cerr << endl;
    if( i==1 ) break;
  }
}


// *************************************************************************
// Class seed_matrix
// Print the matrix
void seed_matrix::print( string outfile ){
  ofstream ofs( outfile.c_str() );
  ofs << n_cols << ":" << n_rows << endl;
  for( unsigned i=n_cols ; ; --i ){
    for( unsigned j=1 ; j<=n_rows ; ++j ){
      coords c = coords::pair( j, i );
      map< coords, int >::iterator it = find( c );
      if( it == end() ){
	cerr << "Error! (" << i << "," << j << ") is not found." << endl;
	exit(-1);
      }
      int sign = it->second;
      if( j!=1 ) ofs << " ";
      if( sign == -1 ){
	ofs << '-';
      } else if( sign == 1 ){
	ofs << '+';
      } else {
	ofs << '.';
      }
    }
    ofs << endl;
    if( i==1 ) break;
  }
  ofs.close();
}


// *************************************************************************
// Class seed_matrix
// Print the wnnso value
void seed_matrix::p_print( string outfile ){
  ofstream ofs( outfile.c_str() );
  for( unsigned i=n_cols ; ; --i ){
    for( unsigned j=1 ; j<=n_rows ; ++j ){
      coords c = coords::pair( i, j );
      map< coords, double >::iterator it = p_find( c );
      if( it == p_end() ){
	cerr << "Error! (" << i << "," << j << ") is not found." << endl;
	exit(-1);
      }
      double p_wnnso = it->second;
      if( j!=1 ) ofs << " ";
      ofs << p_wnnso;
    }
    ofs << endl;
    if( i==1 ) break;
  }
  ofs.close();
}


// *************************************************************************
// Class seed_matrix
// Print the wnnso value
void seed_matrix::r_print( string outfile ){
  ofstream ofs( outfile.c_str() );
  for( unsigned i=n_cols ; ; --i ){
    for( unsigned j=1 ; j<=n_rows ; ++j ){
      coords c = coords::pair( i, j );
      map< coords, double >::iterator it = r_find( c );
      if( it == r_end() ){
	cerr << "Error! (" << i << "," << j << ") is not found." << endl;
	exit(-1);
      }
      double r_wnnso = it->second;
      if( j!=1 ) ofs << " ";
      ofs << r_wnnso;
    }
    ofs << endl;
    if( i==1 ) break;
  }
  ofs.close();
}


// *************************************************************************
// Class seed_matrix
// Draw heat map
void seed_matrix::p_draw( string name, string Rdir ){
  string datfile = name + ".dat";
  string Rfile   = name + ".R";
  string pdffile = name + ".pdf";
  p_print( datfile );
  draw_heatmap( datfile, Rfile, pdffile, n_cols, n_rows, Rdir, true );
}


// *************************************************************************
// Class seed_matrix
// Draw heat map
void seed_matrix::r_draw( string name, string Rdir ){
  string datfile = name + ".dat";
  string Rfile   = name + ".R";
  string pdffile = name + ".pdf";
  r_print( datfile );
  draw_heatmap( datfile, Rfile, pdffile, n_cols, n_rows, Rdir, true );
}
