// *************************************************************************
//  wm_matrix.cpp
// *************************************************************************

#include "wm_matrix.h"
using namespace std;


// *************************************************************************
// Class wm_matrix
// Insert entries
void wm_matrix::insert( mat_idx idx, unsigned dpd, double weight ){
  idx2dpd.insert( map< mat_idx, unsigned >::value_type( idx, dpd ) );
  idx2weight.insert( map< mat_idx, double >::value_type( idx, weight ) );
}


// *************************************************************************
// Class wm_matrix
// Constructor
wm_matrix::wm_matrix( position_matrix& p_mat, unsigned n_cells, 
		      double col_index, double sds ){
  num_cells = n_cells;
  origin = mat_idx::pair( num_cells, num_cells );
  map< mat_idx, string >::iterator p_it = p_mat.idx_begin();
  while( p_it != p_mat.idx_end() ){
    mat_idx  cur_mi = p_it->first;
    unsigned dpd    = p_mat.DPD( origin, cur_mi, true, col_index );
    double   weight = dpd2weight( dpd, sds );
    insert( cur_mi, dpd, weight );
    ++p_it;
  }
}


// *************************************************************************
// Class wm_matrix
// Return DPD value
unsigned wm_matrix::get_dpd( unsigned i, unsigned j ){
  mat_idx mi = mat_idx::pair( i, j );
  return get_dpd( mi );
}


// *************************************************************************
// Class wm_matrix
// Return DPD value
unsigned wm_matrix::get_dpd( mat_idx mi ){
  map< mat_idx, unsigned >::iterator it = d_find( mi );
  if( it == d_end() ){
    cerr << "Error: " << mi.first << ", " << mi.second << " is not found.";
    exit( -1 );
  }
  return it->second;
}


// *************************************************************************
// Class wm_matrix
// Return weight value
double wm_matrix::get_weight( unsigned i, unsigned j ){
  mat_idx mi = mat_idx::pair( i, j );
  return get_weight( mi );
}


// *************************************************************************
// Class wm_matrix
// Return weight value
double wm_matrix::get_weight( mat_idx mi ){
  map< mat_idx, double >::iterator it = w_find( mi );
  if( it == w_end() ){
    cerr << "Error: " << mi.first << ", " << mi.second << " is not found.";
    exit( -1 );
  }
  return it->second;
}


// *************************************************************************
// Class wm_matrix
// Return weight value
double wm_matrix::dpd2weight( unsigned dpd, double sds ){
  double weight = 0.0;
  if( dpd != INF ){
    weight = 1 / ( sds * sqrt( 2 * M_PI ) ) 
      * exp( -1 * (double)dpd / ( 2 * sds * sds ) );
  }
  return weight;
}


// *************************************************************************
// Class wm_matrix
// Show DPD values in this matrix
void wm_matrix::d_display(){
  d_display( num_cells );
}


// *************************************************************************
// Class wm_matrix
// Show DPD values in this matrix
void wm_matrix::d_display( unsigned n_cells ){
  unsigned stt = num_cells - n_cells;
  unsigned end = num_cells + n_cells;
  fprintf( stderr, "%6s   ", "" );
  for( unsigned i=stt ; i<=end ; ++i ){
    int coord_i = (int)i - (int)num_cells;
    fprintf( stderr, "%6d", coord_i );
  }
  cerr << endl;

  fprintf( stderr, "%6s   ", "" );
  for( unsigned i=stt ; i<=end ; ++i ){
    fprintf( stderr, "%6s", "-----" );
  }
  cerr << endl;

  for( unsigned i=end ; ; --i ){
    int coord_i = (int)i - (int)num_cells;
    fprintf( stderr, "%6d | ", coord_i );
    for( unsigned j=stt ; j<=end ; ++j ){
      unsigned dpd = get_dpd( i, j );
      if( dpd == INF ){
	fprintf( stderr, "%6s", "INF" );
      } else {
	fprintf( stderr, "%6d", dpd );
      }
    }
    cerr << endl;
    if( i == stt ) { break; }
  }
}


// *************************************************************************
// Class wm_matrix
// Print DPD values in this matrix
void wm_matrix::d_print( string file_name ){
  ofstream ofs( file_name.c_str() );
  unsigned stt = 0;
  unsigned end = 2 * num_cells;
  for( unsigned i=stt ; i<=end ; ++i ){
    int coord_i = (int)i - (int)num_cells;
    ofs << "\t" << coord_i;
  }
  ofs << endl;

  for( unsigned i=end ;  ; --i ){
    int coord_i = (int)i - (int)num_cells;
    ofs << coord_i;
    for( unsigned j=stt ; j<=end ; ++j ){
      unsigned dpd = get_dpd( i, j );
      if( dpd == INF ){
	ofs << "\t" << "NA";
      } else {
	ofs << "\t" << dpd;
      }
    }
    ofs << endl;
    if( i == stt ) { break; }
  }
}


// *************************************************************************
// Class wm_matrix
// Show weight values in this matrix
void wm_matrix::w_display(){
  w_display( num_cells );
}


// *************************************************************************
// Class wm_matrix
// Show weight values in this matrix
void wm_matrix::w_display( unsigned n_cells ){
  unsigned stt = num_cells - n_cells;
  unsigned end = num_cells + n_cells;
  fprintf( stderr, "%6s   ", "" );
  for( unsigned i=stt ; i<=end ; ++i ){
    int coord_i = (int)i - (int)num_cells;
    fprintf( stderr, "%6d", coord_i );
  }
  cerr << endl;

  fprintf( stderr, "%6s   ", "" );
  for( unsigned i=stt ; i<=end ; ++i ){
    fprintf( stderr, "%6s", "-----" );
  }
  cerr << endl;

  for( unsigned i=end ; ; --i ){
    int coord_i = (int)i - (int)num_cells;
    fprintf( stderr, "%6d | ", coord_i );
    for( unsigned j=stt ; j<=end ; ++j ){
      double weight = get_weight( i, j );
      fprintf( stderr, " %.0e", weight );
    }
    cerr << endl;
    if( i == stt ) { break; }
  }
}


// *************************************************************************
// Class wm_matrix
// Print weight values in this matrix
void wm_matrix::w_print( string file_name ){
  ofstream ofs( file_name.c_str() );
  unsigned stt = 0;
  unsigned end = 2 * num_cells;
  for( unsigned i=stt ; i<=end ; ++i ){
    int coord_i = (int)i - (int)num_cells;
    ofs << "\t" << coord_i;
  }
  ofs << endl;

  for( unsigned i=end ;  ; --i ){
    int coord_i = (int)i - (int)num_cells;
    ofs << coord_i;
    for( unsigned j=stt ; j<=end ; ++j ){
      double weight = get_weight( i, j );
      if( weight == 0 ){
	ofs << "\t" << "NA";
      } else {
	ofs << "\t" << weight;
      }
    }
    ofs << endl;
    if( i == stt ) { break; }
  }
}


// *************************************************************************
// Class wm_matrix
// Draw DPD and weight matrix
void wm_matrix::draw( string jobname, string Rdir ){
  draw_dpd( jobname + ".dpd", Rdir );
  draw_weight( jobname + ".weight", Rdir );
}


// *************************************************************************
// Class wm_matrix
// Draw DPD matrix
void wm_matrix::draw_dpd( string name, string Rdir ){
  string datfile = name + ".dat";
  string Rfile   = name + ".R";
  string pdffile = name + ".pdf";
  d_print( datfile );
  draw_heatmap( datfile, Rfile, pdffile, num_cells, Rdir, false );
}


// *************************************************************************
// Class wm_matrix
// Draw weight matrix
void wm_matrix::draw_weight( string name, string Rdir ){
  string datfile = name + ".dat";
  string Rfile   = name + ".R";
  string pdffile = name + ".pdf";
  w_print( datfile );
  draw_heatmap( datfile, Rfile, pdffile, num_cells, Rdir, true );
}
